@@ -21,7 +21,7 @@
  * COPYING for details.
  *
  */
-  
+
 #ifndef CFG_H_INCLUDED
 #define CFG_H_INCLUDED
 
@@ -61,13 +61,11 @@ struct _GlobalConfig
   /* version number specified by the user, set _after_ parsing is complete */
   /* hex-encoded syslog-ng major/minor, e.g. 0x0201 is syslog-ng 2.1 format */
   gint user_version;
-  
-  /* version number as parsed from the configuration file, it can be set
-   * multiple times if the user uses @version multiple times */
-  gint parsed_version;
+
   const gchar *filename;
   PluginContext plugin_context;
   gboolean use_plugin_discovery;
+  gboolean enable_forced_modules;
   CfgLexer *lexer;
   CfgArgs *globals;
 
@@ -75,7 +73,6 @@ struct _GlobalConfig
   gint mark_freq;
   gint flush_lines;
   gint mark_mode;
-  gint flush_timeout;
   gboolean threaded;
   gboolean pass_unix_credentials;
   gboolean chain_hostnames;
@@ -93,35 +90,43 @@ struct _GlobalConfig
 
   gint log_fifo_size;
   gint log_msg_size;
+  gboolean trim_large_messages;
 
   gboolean create_dirs;
   FilePermOptions file_perm_options;
   GList *source_mangle_callback_list;
   gboolean use_uniqid;
 
-  gboolean keep_timestamp;  
+  gboolean keep_timestamp;
 
   gchar *recv_time_zone;
   LogTemplateOptions template_options;
   HostResolveOptions host_resolve_options;
-  
+
   gchar *file_template_name;
   gchar *proto_template_name;
 
-  gchar *jvm_options;
-  
   LogTemplate *file_template;
   LogTemplate *proto_template;
-  
+
+  guint min_iw_size_per_reader;
+
   PersistConfig *persist;
   PersistState *state;
   GHashTable *module_config;
-  
+
   CfgTree tree;
 
+  GString *preprocess_config;
+  GString *original_config;
+
+  GList *file_list;
 };
 
+gboolean cfg_load_module_with_args(GlobalConfig *cfg, const gchar *module_name, CfgArgs *args);
 gboolean cfg_load_module(GlobalConfig *cfg, const gchar *module_name);
+gboolean cfg_is_module_available(GlobalConfig *self, const gchar *module_name);
+void cfg_discover_candidate_modules(GlobalConfig *self);
 
 Plugin *cfg_find_plugin(GlobalConfig *cfg, gint plugin_type, const gchar *plugin_name);
 gpointer cfg_parse_plugin(GlobalConfig *cfg, Plugin *plugin, YYLTYPE *yylloc, gpointer arg);
@@ -129,23 +134,27 @@ gpointer cfg_parse_plugin(GlobalConfig *cfg, Plugin *plugin, YYLTYPE *yylloc, gp
 gboolean cfg_allow_config_dups(GlobalConfig *self);
 
 void cfg_bad_hostname_set(GlobalConfig *self, gchar *bad_hostname_re);
-gint cfg_lookup_mark_mode(gchar *mark_mode);
-void cfg_set_mark_mode(GlobalConfig *self, gchar *mark_mode);
+gint cfg_lookup_mark_mode(const gchar *mark_mode);
+void cfg_set_mark_mode(GlobalConfig *self, const gchar *mark_mode);
 
 gint cfg_tz_convert_value(gchar *convert);
 gint cfg_ts_format_value(gchar *format);
 
+void cfg_set_version_without_validation(GlobalConfig *self, gint version);
 gboolean cfg_set_version(GlobalConfig *self, gint version);
-void cfg_load_candidate_modules(GlobalConfig *self);
 
 void cfg_set_global_paths(GlobalConfig *self);
 
 GlobalConfig *cfg_new(gint version);
 GlobalConfig *cfg_new_snippet(void);
+GlobalConfig *cfg_new_subordinate(GlobalConfig *master);
 gboolean cfg_run_parser(GlobalConfig *self, CfgLexer *lexer, CfgParser *parser, gpointer *result, gpointer arg);
-gboolean cfg_read_config(GlobalConfig *cfg, const gchar *fname, gboolean syntax_only, gchar *preprocess_into);
-gboolean cfg_load_config(GlobalConfig *self, gchar *config_string, gboolean syntax_only, gchar *preprocess_into);
+gboolean cfg_run_parser_with_main_context(GlobalConfig *self, CfgLexer *lexer, CfgParser *parser, gpointer *result,
+                                          gpointer arg, const gchar *desc);
+gboolean cfg_read_config(GlobalConfig *cfg, const gchar *fname, gchar *preprocess_into);
+void cfg_load_forced_modules(GlobalConfig *self);
 void cfg_shutdown(GlobalConfig *self);
+gboolean cfg_is_shutting_down(GlobalConfig *cfg);
 void cfg_free(GlobalConfig *self);
 gboolean cfg_init(GlobalConfig *cfg);
 gboolean cfg_deinit(GlobalConfig *cfg);
@@ -153,13 +162,14 @@ gboolean cfg_deinit(GlobalConfig *cfg);
 PersistConfig *persist_config_new(void);
 void persist_config_free(PersistConfig *self);
 void cfg_persist_config_move(GlobalConfig *src, GlobalConfig *dest);
-void cfg_persist_config_add(GlobalConfig *cfg, const gchar *name, gpointer value, GDestroyNotify destroy, gboolean force);
+void cfg_persist_config_add(GlobalConfig *cfg, const gchar *name, gpointer value, GDestroyNotify destroy,
+                            gboolean force);
 gpointer cfg_persist_config_fetch(GlobalConfig *cfg, const gchar *name);
 
 typedef gboolean(* mangle_callback)(GlobalConfig *cfg, LogMessage *msg, gpointer user_data);
 
-void register_source_mangle_callback(GlobalConfig *src,mangle_callback cb);
-void uregister_source_mangle_callback(GlobalConfig *src,mangle_callback cb);
+void register_source_mangle_callback(GlobalConfig *src, mangle_callback cb);
+void uregister_source_mangle_callback(GlobalConfig *src, mangle_callback cb);
 
 static inline gboolean
 cfg_is_config_version_older(GlobalConfig *cfg, gint req)
@@ -178,7 +188,20 @@ cfg_set_use_uniqid(gboolean flag)
 }
 
 gint cfg_get_user_version(const GlobalConfig *cfg);
-gint cfg_get_parsed_version(const GlobalConfig *cfg);
-const gchar* cfg_get_filename(const GlobalConfig *cfg);
+guint cfg_get_parsed_version(const GlobalConfig *cfg);
+const gchar *cfg_get_filename(const GlobalConfig *cfg);
+
+static inline EVTTAG *
+cfg_format_version_tag(const gchar *tag_name, gint version)
+{
+  return evt_tag_printf(tag_name, "%d.%d", (version & 0xFF00) >> 8, version & 0xFF);
+}
+
+static inline EVTTAG *
+cfg_format_config_version_tag(GlobalConfig *self)
+{
+  return cfg_format_version_tag("config-version", self->user_version);
+}
+
 
 #endif
