@@ -312,8 +312,8 @@ Test(log_message, test_log_msg_get_value_with_time_related_macro)
   const char *date_value;
 
   msg = log_msg_new_empty();
-  msg->timestamps[LM_TS_STAMP].tv_sec = 1389783444;
-  msg->timestamps[LM_TS_STAMP].zone_offset = 3600;
+  msg->timestamps[LM_TS_STAMP].ut_sec = 1389783444;
+  msg->timestamps[LM_TS_STAMP].ut_gmtoff = 3600;
 
   handle = log_msg_get_value_handle("ISODATE");
   date_value = log_msg_get_value(msg, handle, &value_len);
@@ -326,10 +326,12 @@ Test(log_message, test_local_logmsg_created_with_the_right_flags_and_timestamps)
 {
   LogMessage *msg = log_msg_new_local();
 
-  gboolean are_equals = log_stamp_eq(&msg->timestamps[LM_TS_STAMP], &msg->timestamps[LM_TS_RECVD]);
+  gboolean are_equals = unix_time_eq(&msg->timestamps[LM_TS_STAMP], &msg->timestamps[LM_TS_RECVD]);
 
   cr_assert_neq((msg->flags & LF_LOCAL), 0, "LogMessage created by log_msg_new_local() should have LF_LOCAL flag set");
   cr_assert(are_equals, "The timestamps in a LogMessage created by log_msg_new_local() should be equals");
+
+  log_msg_unref(msg);
 }
 
 Test(log_message, test_sdata_sanitization)
@@ -460,7 +462,6 @@ static void
 test_with_sdata(LogMessage *msg, guint32 old_msg_size)
 {
   sizes_t sizes;
-  gchar key_format[]   = ".SDATA.%02d";
   gchar key[]          = ".SDATA.**";
   gchar value[] = "AAAAAAA";
 
@@ -471,7 +472,7 @@ test_with_sdata(LogMessage *msg, guint32 old_msg_size)
 
   for (char i = 0; i < iter_length; i++)
     {
-      g_sprintf(key, key_format, i);
+      g_sprintf(key, ".SDATA.%02d", i);
       sizes = add_key_value(msg, key, value);
 
       single_sdata_kv_size = NV_ENTRY_DIRECT_HDR + NV_TABLE_BOUND(strlen(key)+1 + strlen(value)+1);
