@@ -23,125 +23,147 @@
  */
 
 #include "logreader.h"
-#include "mainloop-io-worker.h"
 #include "mainloop-call.h"
-#include "ack_tracker.h"
-#include "scratch-buffers.h"
-
-#include <iv_event.h>
-
-struct _LogReader
-{
-  LogSource super;
-  LogProtoServer *proto;
-  gboolean immediate_check;
-  LogPipe *control;
-  LogReaderOptions *options;
-  PollEvents *poll_events;
-  GSockAddr *peer_addr;
-
-  /* NOTE: these used to be LogReaderWatch members, which were merged into
-   * LogReader with the multi-thread refactorization */
-
-  struct iv_task restart_task;
-  struct iv_event schedule_wakeup;
-  struct iv_event last_msg_sent_event;
-  MainLoopIOWorkerJob io_job;
-  gboolean watches_running:1, suspended:1;
-  gint notify_code;
-
-
-  /* proto & poll_events pending to be applied. As long as the previous
-   * processing is being done, we can't replace these in self->proto and
-   * self->poll_events, they get applied to the production ones as soon as
-   * the previous work is finished */
-  gboolean pending_proto_present;
-  GCond *pending_proto_cond;
-  GStaticMutex pending_proto_lock;
-  LogProtoServer *pending_proto;
-  PollEvents *pending_poll_events;
-
-  struct iv_timer idle_timer;
-};
+#include "ack-tracker/ack_tracker.h"
 
+static void log_reader_io_handle_in(gpointer s);
 static gboolean log_reader_fetch_log(LogReader *self);
-
-static void log_reader_stop_watches(LogReader *self);
-static void log_reader_stop_idle_timer(LogReader *self);
-static void log_reader_idle_timeout(void *cookie);
-
 static void log_reader_update_watches(LogReader *self);
 
-static void log_reader_wakeup(LogSource *s);
+/*****************************************************************************
+ * LogReader setters
+ *****************************************************************************/
+
+
+void
+log_reader_set_peer_addr(LogReader *s, GSockAddr *peer_addr)
+{
+  LogReader *self = (LogReader *) s;
+
+  g_sockaddr_unref(self->peer_addr);
+  self->peer_addr = g_sockaddr_ref(peer_addr);
+}
+
+void
+log_reader_set_immediate_check(LogReader *s)
+{
+  LogReader *self = (LogReader *) s;
+
+  self->immediate_check = TRUE;
+}
+
+void
+log_reader_set_options(LogReader *s, LogPipe *control, LogReaderOptions *options,
+                       const gchar *stats_id, const gchar *stats_instance)
+{
+  LogReader *self = (LogReader *) s;
+
+  /* log_reader_reopen() needs to be called prior to set_options.  This is
+   * an ugly hack, but at least it is more explicitly than what used to be
+   * here, which silently ignored if self->proto was NULL.
+   */
+
+  g_assert(self->proto != NULL);
+  gboolean pos_tracked = log_proto_server_is_position_tracked(self->proto);
+
+  log_source_set_options(&self->super, &options->super, stats_id, stats_instance,
+                         (options->flags & LR_THREADED), pos_tracked, control->expr_node);
+
+  log_pipe_unref(self->control);
+  log_pipe_ref(control);
+  self->control = control;
+
+  self->options = options;
+  log_proto_server_set_options(self->proto, &self->options->proto_options.super);
+}
+
+void
+log_reader_set_name(LogReader *self, const gchar *name)
+{
+  log_source_set_name(&self->super, name);
+}
+
+void
+log_reader_disable_bookmark_saving(LogReader *s)
+{
+  log_source_disable_bookmark_saving(&s->super);
+}
+
+/****************************************************************************
+ * Watches: the poll_events instance and the idle timer
+ ***************************************************************************/
 
 static void
-log_reader_apply_proto_and_poll_events(LogReader *self, LogProtoServer *proto, PollEvents *poll_events)
+log_reader_idle_timeout(void *cookie)
 {
-  if (self->proto)
-    log_proto_server_free(self->proto);
+  LogReader *self = (LogReader *) cookie;
+
+  g_assert(!self->io_job.working);
+  msg_notice("Source timeout has elapsed, closing connection",
+             evt_tag_int("fd", log_proto_server_get_fd(self->proto)));
+
+  log_pipe_notify(self->control, NC_CLOSE, self);
+}
+
+static void
+log_reader_stop_idle_timer(LogReader *self)
+{
+  if (iv_timer_registered(&self->idle_timer))
+    iv_timer_unregister(&self->idle_timer);
+}
+
+static void
+log_reader_start_watches(LogReader *self)
+{
+  g_assert(!self->watches_running);
   if (self->poll_events)
-    poll_events_free(self->poll_events);
-
-  self->proto = proto;
-
-  if (self->proto)
-    log_proto_server_set_wakeup_cb(self->proto, (LogProtoServerWakeupFunc) log_reader_wakeup, self);
-
-  self->poll_events = poll_events;
+    poll_events_start_watches(self->poll_events);
+  self->watches_running = TRUE;
+  log_reader_update_watches(self);
 }
 
 static void
-_last_msg_sent(gpointer s)
+log_reader_stop_watches(LogReader *self)
 {
-  LogReader *self = (LogReader *) s;
-  log_pipe_notify(self->control, NC_LAST_MSG_SENT, self);
+  g_assert(self->watches_running);
+  if (self->poll_events)
+    poll_events_stop_watches(self->poll_events);
+  log_reader_stop_idle_timer(self);
+
+  self->watches_running = FALSE;
 }
 
 static void
-log_reader_work_perform(void *s)
+log_reader_disable_watches(LogReader *self)
 {
-  LogReader *self = (LogReader *) s;
+  g_assert(self->watches_running);
+  if (self->poll_events)
+    poll_events_suspend_watches(self->poll_events);
+  log_reader_stop_idle_timer(self);
+}
 
-  self->notify_code = log_reader_fetch_log(self);
+/****************************************************************************
+ * Suspend/wakeup
+ ****************************************************************************/
+
+static void
+log_reader_suspend_until_awoken(LogReader *self)
+{
+  self->immediate_check = FALSE;
+  log_reader_disable_watches(self);
+  self->suspended = TRUE;
 }
 
 static void
-log_reader_work_finished(void *s)
+log_reader_force_check_in_next_poll(LogReader *self)
 {
-  LogReader *self = (LogReader *) s;
+  self->immediate_check = FALSE;
+  log_reader_disable_watches(self);
+  self->suspended = FALSE;
 
-  if (self->pending_proto_present)
+  if (!iv_task_registered(&self->restart_task))
     {
-      /* pending proto is only set in the main thread, so no need to
-       * lock it before coming here. After we're syncing with the
-       * log_writer_reopen() call, quite possibly coming from a
-       * non-main thread. */
-
-      g_static_mutex_lock(&self->pending_proto_lock);
-
-      log_reader_apply_proto_and_poll_events(self, self->pending_proto, self->pending_poll_events);
-      self->pending_proto = NULL;
-      self->pending_poll_events = NULL;
-      self->pending_proto_present = FALSE;
-
-      g_cond_signal(self->pending_proto_cond);
-      g_static_mutex_unlock(&self->pending_proto_lock);
-    }
-
-  if (self->notify_code)
-    {
-      gint notify_code = self->notify_code;
-
-      self->notify_code = 0;
-      log_pipe_notify(self->control, notify_code, self);
-    }
-  if (self->super.super.flags & PIF_INITIALIZED)
-    {
-      /* reenable polling the source assuming that we're still in
-       * business (e.g. the reader hasn't been uninitialized) */
-
-      log_proto_server_reset_error(self->proto);
-      log_reader_update_watches(self);
+      iv_task_register(&self->restart_task);
     }
 }
 
@@ -183,97 +205,71 @@ log_reader_wakeup(LogSource *s)
     iv_event_post(&self->schedule_wakeup);
 }
 
-static void
-log_reader_window_empty(LogSource *s)
-{
-  LogReader *self = (LogReader *) s;
-  if (self->super.super.flags & PIF_INITIALIZED)
-    iv_event_post(&self->last_msg_sent_event);
-}
+/****************************************************************************
+ * Open/close/reopen
+ ***************************************************************************/
 
 static void
-log_reader_io_process_input(gpointer s)
+log_reader_apply_proto_and_poll_events(LogReader *self, LogProtoServer *proto, PollEvents *poll_events)
+{
+  if (self->proto)
+    log_proto_server_free(self->proto);
+  if (self->poll_events)
+    poll_events_free(self->poll_events);
+
+  self->proto = proto;
+
+  if (self->proto)
+    log_proto_server_set_wakeup_cb(self->proto, (LogProtoServerWakeupFunc) log_reader_wakeup, self);
+
+  self->poll_events = poll_events;
+}
+
+
+/* run in the main thread in reaction to a log_reader_reopen to change
+ * the source LogProtoServer instance. It needs to be ran in the main
+ * thread as it reregisters the watches associated with the main
+ * thread. */
+void
+log_reader_close_proto_deferred(gpointer s)
 {
   LogReader *self = (LogReader *) s;
 
+  if (self->io_job.working)
+    {
+      self->pending_close = TRUE;
+      return;
+    }
+
   log_reader_stop_watches(self);
-  if ((self->options->flags & LR_THREADED))
+  log_reader_apply_proto_and_poll_events(self, NULL, NULL);
+  log_reader_start_watches(self);
+}
+
+void
+log_reader_close_proto(LogReader *self)
+{
+  g_assert(self->watches_running);
+  main_loop_call((MainLoopTaskFunc) log_reader_close_proto_deferred, self, TRUE);
+
+  if (!main_loop_is_main_thread())
     {
-      main_loop_io_worker_job_submit(&self->io_job);
-    }
-  else
-    {
-      /* Checking main_loop_io_worker_job_quit() helps to speed up the
-       * reload process.  If reload/shutdown is requested we shouldn't do
-       * anything here, outstanding messages will be processed by the new
-       * configuration.
-       *
-       * Our current understanding is that it doesn't prevent race
-       * conditions of any kind.
-       */
-      if (!main_loop_worker_job_quit())
+      g_static_mutex_lock(&self->pending_close_lock);
+      while (self->pending_close)
         {
-          log_reader_work_perform(s);
-          log_reader_work_finished(s);
+          g_cond_wait(self->pending_close_cond, g_static_mutex_get_mutex(&self->pending_close_lock));
         }
+      g_static_mutex_unlock(&self->pending_close_lock);
     }
 }
 
-static void
-log_reader_init_watches(LogReader *self)
+void
+log_reader_open(LogReader *self, LogProtoServer *proto, PollEvents *poll_events)
 {
-  IV_TASK_INIT(&self->restart_task);
-  self->restart_task.cookie = self;
-  self->restart_task.handler = log_reader_io_process_input;
+  g_assert(!self->watches_running);
+  poll_events_set_callback(poll_events, log_reader_io_handle_in, self);
 
-  IV_EVENT_INIT(&self->schedule_wakeup);
-  self->schedule_wakeup.cookie = self;
-  self->schedule_wakeup.handler = log_reader_wakeup_triggered;
-
-  IV_EVENT_INIT(&self->last_msg_sent_event);
-  self->last_msg_sent_event.cookie = self;
-  self->last_msg_sent_event.handler = _last_msg_sent;
-
-  IV_TIMER_INIT(&self->idle_timer);
-  self->idle_timer.cookie = self;
-  self->idle_timer.handler = log_reader_idle_timeout;
-
-  main_loop_io_worker_job_init(&self->io_job);
-  self->io_job.user_data = self;
-  self->io_job.work = (void (*)(void *)) log_reader_work_perform;
-  self->io_job.completion = (void (*)(void *)) log_reader_work_finished;
-  self->io_job.engage = (void (*)(void *)) log_pipe_ref;
-  self->io_job.release = (void (*)(void *)) log_pipe_unref;
-}
-
-static void
-log_reader_stop_watches(LogReader *self)
-{
-  if (self->watches_running)
-    {
-      poll_events_stop_watches(self->poll_events);
-
-      if (iv_task_registered(&self->restart_task))
-        iv_task_unregister(&self->restart_task);
-      self->watches_running = FALSE;
-    }
-}
-
-static void
-log_reader_stop_idle_timer(LogReader *self)
-{
-  if (iv_timer_registered(&self->idle_timer))
-    iv_timer_unregister(&self->idle_timer);
-}
-
-static void
-log_reader_start_watches_if_stopped(LogReader *self)
-{
-  if (!self->watches_running)
-    {
-      poll_events_start_watches(self->poll_events);
-      self->watches_running = TRUE;
-    }
+  log_reader_apply_proto_and_poll_events(self, proto, poll_events);
 }
 
 static gboolean
@@ -282,26 +278,9 @@ log_reader_is_opened(LogReader *self)
   return self->proto && self->poll_events;
 }
 
-static void
-log_reader_suspend_until_awoken(LogReader *self)
-{
-  self->immediate_check = FALSE;
-  poll_events_suspend_watches(self->poll_events);
-  self->suspended = TRUE;
-}
-
-static void
-log_reader_force_check_in_next_poll(LogReader *self)
-{
-  self->immediate_check = FALSE;
-  poll_events_suspend_watches(self->poll_events);
-  self->suspended = FALSE;
-
-  if (!iv_task_registered(&self->restart_task))
-    {
-      iv_task_register(&self->restart_task);
-    }
-}
+/*****************************************************************************
+ * Set watches state so we are polling the event(s) that comes next.
+ *****************************************************************************/
 
 static void
 log_reader_update_watches(LogReader *self)
@@ -310,14 +289,13 @@ log_reader_update_watches(LogReader *self)
   gint idle_timeout = -1;
 
   main_loop_assert_main_thread();
+  g_assert(self->watches_running);
 
-  log_reader_stop_idle_timer(self);
+  log_reader_disable_watches(self);
 
   if (!log_reader_is_opened(self))
     return;
 
-  log_reader_start_watches_if_stopped(self);
-
   gboolean free_to_send = log_source_free_to_send(&self->super);
   if (!free_to_send)
     {
@@ -360,6 +338,65 @@ log_reader_update_watches(LogReader *self)
     }
 }
 
+/*****************************************************************************
+ * Glue into MainLoopIOWorker
+ *****************************************************************************/
+
+static void
+log_reader_work_perform(void *s, GIOCondition cond)
+{
+  LogReader *self = (LogReader *) s;
+
+  self->notify_code = log_reader_fetch_log(self);
+}
+
+static void
+log_reader_work_finished(void *s)
+{
+  LogReader *self = (LogReader *) s;
+
+  if (self->pending_close)
+    {
+      /* pending proto is only set in the main thread, so no need to
+       * lock it before coming here. After we're syncing with the
+       * log_writer_reopen() call, quite possibly coming from a
+       * non-main thread. */
+
+      g_static_mutex_lock(&self->pending_close_lock);
+
+      log_reader_apply_proto_and_poll_events(self, NULL, NULL);
+      self->pending_close = FALSE;
+
+      g_cond_signal(self->pending_close_cond);
+      g_static_mutex_unlock(&self->pending_close_lock);
+    }
+
+  if (self->notify_code)
+    {
+      gint notify_code = self->notify_code;
+
+      self->notify_code = 0;
+      log_pipe_notify(self->control, notify_code, self);
+    }
+  if (self->super.super.flags & PIF_INITIALIZED)
+    {
+      /* reenable polling the source assuming that we're still in
+       * business (e.g. the reader hasn't been uninitialized) */
+
+      if (self->realloc_window_after_fetch)
+        {
+          self->realloc_window_after_fetch = FALSE;
+          log_source_dynamic_window_realloc(s);
+        }
+      log_proto_server_reset_error(self->proto);
+      log_reader_update_watches(self);
+    }
+}
+
+/*****************************************************************************
+ * Input processing, the main function of LogReader
+ *****************************************************************************/
+
 static void
 _add_aux_nvpair(const gchar *name, const gchar *value, gsize value_len, gpointer user_data)
 {
@@ -468,15 +505,11 @@ log_reader_fetch_log(LogReader *self)
         {
           msg_count++;
 
-          ScratchBuffersMarker mark;
-          scratch_buffers_mark(&mark);
           if (!log_reader_handle_line(self, msg, msg_len, &aux))
             {
-              scratch_buffers_reclaim_marked(mark);
               /* window is full, don't generate further messages */
               break;
             }
-          scratch_buffers_reclaim_marked(mark);
         }
     }
   log_transport_aux_data_destroy(&aux);
@@ -486,6 +519,38 @@ log_reader_fetch_log(LogReader *self)
   return 0;
 }
 
+static void
+log_reader_io_handle_in(gpointer s)
+{
+  LogReader *self = (LogReader *) s;
+
+  log_reader_disable_watches(self);
+  if ((self->options->flags & LR_THREADED))
+    {
+      main_loop_io_worker_job_submit(&self->io_job, G_IO_IN);
+    }
+  else
+    {
+      /* Checking main_loop_io_worker_job_quit() helps to speed up the
+       * reload process.  If reload/shutdown is requested we shouldn't do
+       * anything here, outstanding messages will be processed by the new
+       * configuration.
+       *
+       * Our current understanding is that it doesn't prevent race
+       * conditions of any kind.
+       */
+      if (!main_loop_worker_job_quit())
+        {
+          log_reader_work_perform(s, G_IO_IN);
+          log_reader_work_finished(s);
+        }
+    }
+}
+
+/*****************************************************************************
+ * LogReader->LogPipe interface implementation
+ *****************************************************************************/
+
 static gboolean
 log_reader_init(LogPipe *s)
 {
@@ -494,6 +559,9 @@ log_reader_init(LogPipe *s)
   if (!log_source_init(s))
     return FALSE;
 
+  log_proto_server_set_ack_tracker(self->proto, self->super.ack_tracker);
+
+
   if (!log_proto_server_validate_options(self->proto))
     return FALSE;
 
@@ -504,11 +572,9 @@ log_reader_init(LogPipe *s)
       return FALSE;
     }
 
-  poll_events_set_callback(self->poll_events, log_reader_io_process_input, self);
-
-  log_reader_update_watches(self);
   iv_event_register(&self->schedule_wakeup);
-  iv_event_register(&self->last_msg_sent_event);
+
+  log_reader_start_watches(self);
 
   return TRUE;
 }
@@ -521,9 +587,10 @@ log_reader_deinit(LogPipe *s)
   main_loop_assert_main_thread();
 
   iv_event_unregister(&self->schedule_wakeup);
-  iv_event_unregister(&self->last_msg_sent_event);
+  if (iv_task_registered(&self->restart_task))
+    iv_task_unregister(&self->restart_task);
+
   log_reader_stop_watches(self);
-  log_reader_stop_idle_timer(self);
 
   if (!log_source_deinit(s))
     return FALSE;
@@ -531,6 +598,30 @@ log_reader_deinit(LogPipe *s)
   return TRUE;
 }
 
+
+static void
+log_reader_init_watches(LogReader *self)
+{
+  IV_TASK_INIT(&self->restart_task);
+  self->restart_task.cookie = self;
+  self->restart_task.handler = log_reader_io_handle_in;
+
+  IV_EVENT_INIT(&self->schedule_wakeup);
+  self->schedule_wakeup.cookie = self;
+  self->schedule_wakeup.handler = log_reader_wakeup_triggered;
+
+  IV_TIMER_INIT(&self->idle_timer);
+  self->idle_timer.cookie = self;
+  self->idle_timer.handler = log_reader_idle_timeout;
+
+  main_loop_io_worker_job_init(&self->io_job);
+  self->io_job.user_data = self;
+  self->io_job.work = (void (*)(void *, GIOCondition)) log_reader_work_perform;
+  self->io_job.completion = (void (*)(void *)) log_reader_work_finished;
+  self->io_job.engage = (void (*)(void *)) log_pipe_ref;
+  self->io_job.release = (void (*)(void *)) log_pipe_unref;
+}
+
 static void
 log_reader_free(LogPipe *s)
 {
@@ -546,105 +637,25 @@ log_reader_free(LogPipe *s)
 
   log_pipe_unref(self->control);
   g_sockaddr_unref(self->peer_addr);
-  g_static_mutex_free(&self->pending_proto_lock);
-  g_cond_free(self->pending_proto_cond);
+  g_static_mutex_free(&self->pending_close_lock);
+  g_cond_free(self->pending_close_cond);
   log_source_free(s);
 }
 
-void
-log_reader_set_options(LogReader *s, LogPipe *control, LogReaderOptions *options,
-                       const gchar *stats_id, const gchar *stats_instance)
+static void
+_schedule_dynamic_window_realloc(LogSource *s)
 {
-  LogReader *self = (LogReader *) s;
+  LogReader *self = (LogReader *)s;
 
-  /* log_reader_reopen() needs to be called prior to set_options.  This is
-   * an ugly hack, but at least it is more explicitly than what used to be
-   * here, which silently ignored if self->proto was NULL.
-   */
-
-  g_assert(self->proto != NULL);
-  gboolean pos_tracked = log_proto_server_is_position_tracked(self->proto);
-
-  log_source_set_options(&self->super, &options->super, stats_id, stats_instance,
-                         (options->flags & LR_THREADED), pos_tracked, control->expr_node);
-
-  log_pipe_unref(self->control);
-  log_pipe_ref(control);
-  self->control = control;
-
-  self->options = options;
-  log_proto_server_set_options(self->proto, &self->options->proto_options.super);
-  log_proto_server_set_ack_tracker(self->proto, self->super.ack_tracker);
-}
-
-/* run in the main thread in reaction to a log_reader_reopen to change
- * the source LogProtoServer instance. It needs to be ran in the main
- * thread as it reregisters the watches associated with the main
- * thread. */
-void
-log_reader_reopen_deferred(gpointer s)
-{
-  gpointer *args = (gpointer *) s;
-  LogReader *self = args[0];
-  LogProtoServer *proto = args[1];
-  PollEvents *poll_events = args[2];
+  msg_trace("LogReader::dynamic_window_realloc called");
 
   if (self->io_job.working)
     {
-      self->pending_proto = proto;
-      self->pending_poll_events = poll_events;
-      self->pending_proto_present = TRUE;
+      self->realloc_window_after_fetch = TRUE;
       return;
     }
 
-  log_reader_stop_watches(self);
-  log_reader_stop_idle_timer(self);
-  log_reader_apply_proto_and_poll_events(self, proto, poll_events);
-  log_reader_update_watches(self);
-}
-
-void
-log_reader_reopen(LogReader *self, LogProtoServer *proto, PollEvents *poll_events)
-{
-  gpointer args[] = { self, proto, poll_events };
-
-  main_loop_call((MainLoopTaskFunc) log_reader_reopen_deferred, args, TRUE);
-
-  if (!main_loop_is_main_thread())
-    {
-      g_static_mutex_lock(&self->pending_proto_lock);
-      while (self->pending_proto_present)
-        {
-          g_cond_wait(self->pending_proto_cond, g_static_mutex_get_mutex(&self->pending_proto_lock));
-        }
-      g_static_mutex_unlock(&self->pending_proto_lock);
-    }
-}
-
-void
-log_reader_close_proto(LogReader *self)
-{
-  log_reader_reopen(self, NULL, NULL);
-}
-
-static void
-log_reader_idle_timeout(void *cookie)
-{
-  LogReader *self = (LogReader *) cookie;
-
-  msg_notice("Source timeout has elapsed, closing connection",
-             evt_tag_int("fd", log_proto_server_get_fd(self->proto)));
-
-  log_pipe_notify(self->control, NC_CLOSE, self);
-}
-
-void
-log_reader_set_peer_addr(LogReader *s, GSockAddr *peer_addr)
-{
-  LogReader *self = (LogReader *) s;
-
-  g_sockaddr_unref(self->peer_addr);
-  self->peer_addr = g_sockaddr_ref(peer_addr);
+  log_source_dynamic_window_realloc(s);
 }
 
 LogReader *
@@ -657,21 +668,17 @@ log_reader_new(GlobalConfig *cfg)
   self->super.super.deinit = log_reader_deinit;
   self->super.super.free_fn = log_reader_free;
   self->super.wakeup = log_reader_wakeup;
-  self->super.window_empty_cb = log_reader_window_empty;
+  self->super.schedule_dynamic_window_realloc = _schedule_dynamic_window_realloc;
   self->immediate_check = FALSE;
   log_reader_init_watches(self);
-  g_static_mutex_init(&self->pending_proto_lock);
-  self->pending_proto_cond = g_cond_new();
+  g_static_mutex_init(&self->pending_close_lock);
+  self->pending_close_cond = g_cond_new();
   return self;
 }
 
-void
-log_reader_set_immediate_check(LogReader *s)
-{
-  LogReader *self = (LogReader *) s;
-
-  self->immediate_check = TRUE;
-}
+/****************************************************************************
+ * LogReaderOptions defaults/init/destroy
+ ***************************************************************************/
 
 void
 log_reader_options_defaults(LogReaderOptions *options)
