@@ -58,6 +58,7 @@ path_resolver_populate_configure_variables(PathResolver *self, const gchar *sysp
   path_resolver_add_configure_variable(&self->super, "${datarootdir}", SYSLOG_NG_PATH_DATAROOTDIR);
   path_resolver_add_configure_variable(&self->super, "${datadir}", SYSLOG_NG_PATH_DATADIR);
   path_resolver_add_configure_variable(&self->super, "${localstatedir}", SYSLOG_NG_PATH_LOCALSTATEDIR);
+  path_resolver_add_configure_variable(&self->super, "${moduledir}", SYSLOG_NG_PATH_MODULEDIR);
 }
 
 
@@ -161,7 +162,7 @@ path_resolver_new(const gchar *sysprefix)
   return &self->super;
 }
 
-static gchar *
+static const gchar *
 lookup_sysprefix(void)
 {
   gchar *v;
