@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2012 Balabit
+ * Copyright (c) 2012-2019 Balabit
  * Copyright (c) 2012 Balázs Scheidler
  *
  * This library is free software; you can redistribute it and/or
@@ -25,6 +25,8 @@
 #include "msg_parse_lib.h"
 #include "plugin.h"
 
+#include <criterion/criterion.h>
+
 MsgFormatOptions parse_options;
 
 void
@@ -47,13 +49,13 @@ deinit_syslogformat_module(void)
 void
 assert_log_message_has_tag(LogMessage *log_message, const gchar *tag_name)
 {
-  assert_true(log_msg_is_tag_by_name(log_message, tag_name), "Expected message to have '%s' tag", tag_name);
+  cr_assert(log_msg_is_tag_by_name(log_message, tag_name), "Expected message to have '%s' tag", tag_name);
 }
 
 void
 assert_log_message_doesnt_have_tag(LogMessage *log_message, const gchar *tag_name)
 {
-  assert_false(log_msg_is_tag_by_name(log_message, tag_name), "Expected message not to have '%s' tag", tag_name);
+  cr_assert_not(log_msg_is_tag_by_name(log_message, tag_name), "Expected message not to have '%s' tag", tag_name);
 }
 
 void
@@ -65,9 +67,12 @@ assert_log_message_value(LogMessage *self, NVHandle handle, const gchar *expecte
   const gchar *actual_value = log_msg_get_value(self, handle, &value_length);
 
   if (expected_value)
-    assert_string(actual_value, expected_value, "Value is not expected for key %s", key_name);
+    {
+      cr_assert_str_eq(actual_value, expected_value, "Invalid value for key %s; actual: %s, expected: %s", key_name,
+                       actual_value, expected_value);
+    }
   else
-    assert_string(actual_value, "", "No value is expected for key %s but its value is %s", key_name, actual_value);
+    cr_assert_str_eq(actual_value, "", "No value is expected for key %s but its value is %s", key_name, actual_value);
 }
 
 void
@@ -83,7 +88,7 @@ assert_log_messages_saddr(LogMessage *log_message_a, LogMessage *log_message_b)
 
   g_sockaddr_format(log_message_a->saddr, address_a, sizeof(address_a), GSA_FULL);
   g_sockaddr_format(log_message_b->saddr, address_b, sizeof(address_b), GSA_FULL);
-  assert_string(address_a, address_b, "Socket address is not expected");
+  cr_assert_str_eq(address_a, address_b, "Non-matching socket address; a: %s, b: %s", address_a, address_b);
 }
 
 void
@@ -94,7 +99,9 @@ assert_structured_data_of_messages(LogMessage *log_message_a, LogMessage *log_me
 
   log_msg_format_sdata(log_message_a, structured_data_string_a, 0);
   log_msg_format_sdata(log_message_b, structured_data_string_b, 0);
-  assert_string(structured_data_string_a->str, structured_data_string_b->str, "Structure data string are not the same");
+  cr_assert_str_eq(structured_data_string_a->str, structured_data_string_b->str,
+                   "Structure data string are not the same; a: %s, b: %s",
+                   structured_data_string_a->str, structured_data_string_b->str);
 
   g_string_free(structured_data_string_a, TRUE);
   g_string_free(structured_data_string_b, TRUE);
@@ -108,20 +115,20 @@ assert_log_message_values_equal(LogMessage *log_message_a, LogMessage *log_messa
   const gchar *value_a = log_msg_get_value(log_message_a, handle, NULL);
   const gchar *value_b = log_msg_get_value(log_message_b, handle, NULL);
 
-  assert_string(value_a, value_b, "Value is not expected for key %s", key_name);
+  cr_assert_str_eq(value_a, value_b, "Non-matching value for key %s; a: %s, b: %s", key_name, value_a, value_b);
 }
 
 void
 assert_log_messages_equal(LogMessage *log_message_a, LogMessage *log_message_b)
 {
-  assert_gint(log_message_a->timestamps[LM_TS_STAMP].tv_sec, log_message_b->timestamps[LM_TS_STAMP].tv_sec,
-              "Timestamps are not the same");
-  assert_guint32(log_message_a->timestamps[LM_TS_STAMP].tv_usec, log_message_b->timestamps[LM_TS_STAMP].tv_usec,
-                 "Timestamps usec are not the same");
-  assert_guint32(log_message_a->timestamps[LM_TS_STAMP].zone_offset, log_message_b->timestamps[LM_TS_STAMP].zone_offset,
-                 "Timestamp offset are not the same");
+  cr_assert_eq(log_message_a->timestamps[LM_TS_STAMP].ut_sec, log_message_b->timestamps[LM_TS_STAMP].ut_sec,
+               "Timestamps are not the same");
+  cr_assert_eq(log_message_a->timestamps[LM_TS_STAMP].ut_usec, log_message_b->timestamps[LM_TS_STAMP].ut_usec,
+               "Timestamps usec are not the same");
+  cr_assert_eq(log_message_a->timestamps[LM_TS_STAMP].ut_gmtoff, log_message_b->timestamps[LM_TS_STAMP].ut_gmtoff,
+               "Timestamp offset are not the same");
 
-  assert_guint16(log_message_a->pri, log_message_b->pri, "Priorities are not the same");
+  cr_assert_eq(log_message_a->pri, log_message_b->pri, "Priorities are not the same");
 
   assert_log_message_values_equal(log_message_a, log_message_b, LM_V_HOST);
   assert_log_message_values_equal(log_message_a, log_message_b, LM_V_PROGRAM);
