@@ -360,7 +360,7 @@ _vp_process_value(const gchar *name, const gchar *prefix, TypeHint type,
   return FALSE;
 }
 
-static worker_insert_result_t
+static LogThreadedResult
 _worker_insert(LogThreadedDestDriver *s, LogMessage *msg)
 {
   MongoDBDestDriver *self = (MongoDBDestDriver *)s;
@@ -368,7 +368,7 @@ _worker_insert(LogThreadedDestDriver *s, LogMessage *msg)
   gboolean drop_silently = self->template_options.on_error & ON_ERROR_SILENT;
 
   if (!_connect(self, TRUE))
-    return WORKER_INSERT_RESULT_NOT_CONNECTED;
+    return LTR_NOT_CONNECTED;
 
   bson_reinit(self->bson);
 
@@ -390,7 +390,7 @@ _worker_insert(LogThreadedDestDriver *s, LogMessage *msg)
                                         LTZ_SEND, &self->template_options),
                     evt_tag_str("driver", self->super.super.super.id));
         }
-      return WORKER_INSERT_RESULT_DROP;
+      return LTR_DROP;
     }
 
   msg_debug("Outgoing message to MongoDB destination",
@@ -409,7 +409,7 @@ _worker_insert(LogThreadedDestDriver *s, LogMessage *msg)
                     evt_tag_int("time_reopen", self->super.time_reopen),
                     evt_tag_str("reason", error.message),
                     evt_tag_str("driver", self->super.super.super.id));
-          return WORKER_INSERT_RESULT_NOT_CONNECTED;
+          return LTR_NOT_CONNECTED;
         }
       else
         {
@@ -417,11 +417,11 @@ _worker_insert(LogThreadedDestDriver *s, LogMessage *msg)
                     evt_tag_int("time_reopen", self->super.time_reopen),
                     evt_tag_str("reason", error.message),
                     evt_tag_str("driver", self->super.super.super.id));
-          return WORKER_INSERT_RESULT_ERROR;
+          return LTR_ERROR;
         }
     }
 
-  return WORKER_INSERT_RESULT_SUCCESS;
+  return LTR_SUCCESS;
 }
 
 gboolean
@@ -574,7 +574,7 @@ afmongodb_dd_new(GlobalConfig *cfg)
   self->super.worker.disconnect = _worker_disconnect;
   self->super.worker.insert = _worker_insert;
   self->super.format_stats_instance = _format_stats_instance;
-  self->super.stats_source = SCS_MONGODB;
+  self->super.stats_source = stats_register_type("mongodb");
 
 #if SYSLOG_NG_ENABLE_LEGACY_MONGODB_OPTIONS
   afmongodb_dd_init_legacy(self);
