@@ -28,6 +28,7 @@
 #include "stats/stats-registry.h"
 #include "mainloop.h"
 #include "poll-fd-events.h"
+#include "timeutils/misc.h"
 
 #include <string.h>
 #include <sys/types.h>
@@ -39,6 +40,9 @@ int allow_severity = 0;
 int deny_severity = 0;
 #endif
 
+static const gfloat DYNAMIC_WINDOW_TIMER_MSECS = 1000;
+static const gsize DYNAMIC_WINDOW_REALLOC_TICKS = 5;
+
 typedef struct _AFSocketSourceConnection
 {
   LogPipe super;
@@ -51,7 +55,7 @@ typedef struct _AFSocketSourceConnection
 static void afsocket_sd_close_connection(AFSocketSourceDriver *self, AFSocketSourceConnection *sc);
 
 static gchar *
-afsocket_sc_stats_instance(AFSocketSourceConnection *self)
+_format_sc_name(AFSocketSourceConnection *self, gint format_type)
 {
   static gchar buf[256];
   gchar peer_addr[MAX_SOCKADDR_STRING];
@@ -61,18 +65,30 @@ afsocket_sc_stats_instance(AFSocketSourceConnection *self)
       /* dgram connection, which means we have no peer, use the bind address */
       if (self->owner->bind_addr)
         {
-          g_sockaddr_format(self->owner->bind_addr, buf, sizeof(buf), GSA_ADDRESS_ONLY);
+          g_sockaddr_format(self->owner->bind_addr, buf, sizeof(buf), format_type);
           return buf;
         }
       else
         return NULL;
     }
 
-  g_sockaddr_format(self->peer_addr, peer_addr, sizeof(peer_addr), GSA_ADDRESS_ONLY);
+  g_sockaddr_format(self->peer_addr, peer_addr, sizeof(peer_addr), format_type);
   g_snprintf(buf, sizeof(buf), "%s,%s", self->owner->transport_mapper->transport, peer_addr);
   return buf;
 }
 
+static gchar *
+afsocket_sc_stats_instance(AFSocketSourceConnection *self)
+{
+  return _format_sc_name(self, GSA_ADDRESS_ONLY);
+}
+
+static gchar *
+afsocket_sc_format_name(AFSocketSourceConnection *self)
+{
+  return _format_sc_name(self, GSA_FULL);
+}
+
 static LogTransport *
 afsocket_sc_construct_transport(AFSocketSourceConnection *self, gint fd)
 {
@@ -86,7 +102,8 @@ afsocket_sc_init(LogPipe *s)
   LogTransport *transport;
   LogProtoServer *proto;
 
-  if (!self->reader)
+  gboolean restored_kept_alive_source = !!self->reader;
+  if (!restored_kept_alive_source)
     {
       transport = afsocket_sc_construct_transport(self, self->sock);
       /* transport_mapper_inet_construct_log_transport() can return NULL on TLS errors */
@@ -102,13 +119,18 @@ afsocket_sc_init(LogPipe *s)
         }
 
       self->reader = log_reader_new(s->cfg);
-      log_reader_reopen(self->reader, proto, poll_fd_events_new(self->sock));
+      log_reader_open(self->reader, proto, poll_fd_events_new(self->sock));
       log_reader_set_peer_addr(self->reader, self->peer_addr);
     }
+
   log_reader_set_options(self->reader, &self->super,
                          &self->owner->reader_options,
                          self->owner->super.super.id,
                          afsocket_sc_stats_instance(self));
+  log_reader_set_name(self->reader, afsocket_sc_format_name(self));
+
+  if (!restored_kept_alive_source && self->owner->dynamic_window_pool)
+    log_source_enable_dynamic_window(&self->reader->super, self->owner->dynamic_window_pool);
 
   log_pipe_append((LogPipe *) self->reader, s);
   if (log_pipe_init((LogPipe *) self->reader))
@@ -268,6 +290,30 @@ afsocket_sd_set_listen_backlog(LogDriver *s, gint listen_backlog)
   self->listen_backlog = listen_backlog;
 }
 
+void
+afsocket_sd_set_dynamic_window_size(LogDriver *s, gint dynamic_window_size)
+{
+  AFSocketSourceDriver *self = (AFSocketSourceDriver *) s;
+
+  self->dynamic_window_size = dynamic_window_size;
+}
+
+void
+afsocket_sd_set_dynamic_window_stats_freq(LogDriver *s, gfloat stats_freq)
+{
+  AFSocketSourceDriver *self = (AFSocketSourceDriver *) s;
+
+  self->dynamic_window_stats_freq = stats_freq * 1000.0f;
+}
+
+void
+afsocket_sd_set_dynamic_window_realloc_ticks(LogDriver *s, gint realloc_ticks)
+{
+  AFSocketSourceDriver *self = (AFSocketSourceDriver *) s;
+
+  self->dynamic_window_realloc_ticks = realloc_ticks;
+}
+
 static const gchar *
 afsocket_sd_format_name(const LogPipe *s)
 {
@@ -313,6 +359,17 @@ afsocket_sd_format_connections_name(const AFSocketSourceDriver *self)
   return persist_name;
 }
 
+static const gchar *
+afsocket_sd_format_dynamic_window_pool_name(const AFSocketSourceDriver *self)
+{
+  static gchar persist_name[1024];
+
+  g_snprintf(persist_name, sizeof(persist_name), "%s.dynamic_window",
+             afsocket_sd_format_name((const LogPipe *)self));
+
+  return persist_name;
+}
+
 static gboolean
 afsocket_sd_process_connection(AFSocketSourceDriver *self, GSockAddr *client_addr, GSockAddr *local_addr, gint fd)
 {
@@ -451,20 +508,170 @@ afsocket_sd_close_connection(AFSocketSourceDriver *self, AFSocketSourceConnectio
 }
 
 static void
-afsocket_sd_start_watches(AFSocketSourceDriver *self)
+_listen_fd_init(AFSocketSourceDriver *self)
 {
   IV_FD_INIT(&self->listen_fd);
   self->listen_fd.fd = self->fd;
   self->listen_fd.cookie = self;
   self->listen_fd.handler_in = afsocket_sd_accept;
+}
+
+static void
+_listen_fd_start(AFSocketSourceDriver *self)
+{
   iv_fd_register(&self->listen_fd);
 }
 
+static void
+_dynamic_window_timer_start(AFSocketSourceDriver *self)
+{
+  iv_validate_now();
+  self->dynamic_window_timer.expires = iv_now;
+  timespec_add_msec(&self->dynamic_window_timer.expires, self->dynamic_window_stats_freq);
+  iv_timer_register(&self->dynamic_window_timer);
+}
+
+static void
+_log_source_dynamic_window_update_statistics_cb(AFSocketSourceConnection *conn)
+{
+  log_source_dynamic_window_update_statistics(&conn->reader->super);
+}
+
+static void
+_dynamic_window_update_stats(AFSocketSourceDriver *self)
+{
+  g_list_foreach(self->connections, (GFunc) _log_source_dynamic_window_update_statistics_cb, NULL);
+}
+
+static void
+_log_source_dynamic_window_realloc_cb(AFSocketSourceConnection *conn)
+{
+  log_source_schedule_dynamic_window_realloc(&conn->reader->super);
+}
+
+static void
+_dynamic_window_realloc(AFSocketSourceDriver *self)
+{
+  g_list_foreach(self->connections, (GFunc) _log_source_dynamic_window_realloc_cb, NULL);
+}
+
+static void
+_dynamic_window_set_balanced_window(AFSocketSourceDriver *self)
+{
+  if (self->num_connections <= 0)
+    return;
+
+  gsize new_balanced_win = self->dynamic_window_pool->pool_size / self->num_connections;
+  if (new_balanced_win == 0)
+    {
+      msg_info("Cannot allocate more dynamic window for new clients. From now, only static window is allocated."
+               "The reason of dynamic-window-pool exhaustion is that the number of clients is larger than the"
+               " dynamic-window-size",
+               evt_tag_long("total_dynamic_window_size", self->dynamic_window_size),
+               evt_tag_int("max_connections", self->max_connections),
+               evt_tag_int("active_connections", self->num_connections),
+               evt_tag_long("dynamic_window_size_for_existing_clients", self->dynamic_window_pool->balanced_window),
+               evt_tag_long("static_window_size", self->reader_options.super.init_window_size));
+      return;
+    }
+
+  self->dynamic_window_pool->balanced_window = new_balanced_win;
+}
+
+static gboolean
+_is_dynamic_window_realloc_needed(AFSocketSourceDriver *self)
+{
+  return self->dynamic_window_timer_tick >= self->dynamic_window_realloc_ticks;
+}
+
+static void
+_on_dynamic_window_timer_elapsed(gpointer cookie)
+{
+  AFSocketSourceDriver *self = (AFSocketSourceDriver *)cookie;
+
+  if (_is_dynamic_window_realloc_needed(self))
+    {
+      _dynamic_window_set_balanced_window(self);
+      _dynamic_window_realloc(self);
+      self->dynamic_window_timer_tick = 0;
+    }
+  else
+    {
+      _dynamic_window_update_stats(self);
+    }
+
+  self->dynamic_window_timer_tick++;
+
+  msg_trace("Dynamic window timer elapsed", evt_tag_int("tick", self->dynamic_window_timer_tick));
+  _dynamic_window_timer_start(self);
+}
+
+static void
+_dynamic_window_timer_init(AFSocketSourceDriver *self)
+{
+  IV_TIMER_INIT(&self->dynamic_window_timer);
+  self->dynamic_window_timer.cookie = self;
+  self->dynamic_window_timer.handler = _on_dynamic_window_timer_elapsed;
+}
+
+static void
+_dynamic_window_timer_stop(AFSocketSourceDriver *self)
+{
+  if (iv_timer_registered(&self->dynamic_window_timer))
+    iv_timer_unregister(&self->dynamic_window_timer);
+}
+static void
+_listen_fd_stop(AFSocketSourceDriver *self)
+{
+  if (iv_fd_registered (&self->listen_fd))
+    iv_fd_unregister(&self->listen_fd);
+}
+
+static void
+_init_watches(AFSocketSourceDriver *self)
+{
+  _dynamic_window_timer_init(self);
+}
+
+static void
+afsocket_sd_start_watches(AFSocketSourceDriver *self)
+{
+  _listen_fd_start(self);
+
+  if (self->dynamic_window_pool != NULL)
+    _dynamic_window_timer_start(self);
+}
+
 static void
 afsocket_sd_stop_watches(AFSocketSourceDriver *self)
 {
-  if (iv_fd_registered (&self->listen_fd))
-    iv_fd_unregister(&self->listen_fd);
+  _listen_fd_stop(self);
+  _dynamic_window_timer_stop(self);
+}
+
+static void
+_adjust_dynamic_window_size_if_needed(AFSocketSourceDriver *self)
+{
+  if (self->max_connections > 0 && self->dynamic_window_size > 0)
+    {
+      gint remainder = self->dynamic_window_size % self->max_connections;
+      if (remainder)
+        {
+          gsize new_dynamic_window_size = self->dynamic_window_size + (self->max_connections - remainder);
+          msg_warning("WARNING: dynamic-window-size() is advised to be a multiple of max-connections(), "
+                      "to achieve effective dynamic-window usage. Adjusting dynamic-window-size()",
+                      evt_tag_int("orig_dynamic_window_size", self->dynamic_window_size),
+                      evt_tag_int("new_dynamic_window_size", new_dynamic_window_size),
+                      log_pipe_location_tag(&self->super.super.super));
+          self->dynamic_window_size = new_dynamic_window_size;
+        }
+      if (self->dynamic_window_size / self->max_connections < 10)
+        {
+          msg_warning("WARNING: dynamic-window-size() is advised to be at least 10 times larger, than max-connections(), "
+                      "to achieve effective dynamic-window usage. Please update your configuration",
+                      log_pipe_location_tag(&self->super.super.super));
+        }
+    }
 }
 
 static gboolean
@@ -479,16 +686,36 @@ afsocket_sd_setup_reader_options(AFSocketSourceDriver *self)
        * window sizes. Increase that but warn the user at the same time.
        */
 
+      /* NOTE: this changes the initial window size from 100 to 1000. Reasons:
+      * Starting with syslog-ng 3.3, window-size is distributed evenly between
+      * _all_ possible connections to avoid starving.  With the defaults this
+      * means that we get a window size of 10 messages log_iw_size(100) /
+      * max_connections(10), but that is incredibly slow, thus bump this value here.
+      */
+
+      if (self->reader_options.super.init_window_size == -1)
+        {
+          self->reader_options.super.init_window_size = 1000;
+          if (self->dynamic_window_size != 0)
+            self->reader_options.super.init_window_size = self->max_connections * 10;
+        }
+
+      guint min_iw_size_per_reader = cfg->min_iw_size_per_reader;
+      if (self->dynamic_window_size != 0)
+        min_iw_size_per_reader = 1;
+
+      _adjust_dynamic_window_size_if_needed(self);
+
       self->reader_options.super.init_window_size /= self->max_connections;
-      if (self->reader_options.super.init_window_size < cfg->min_iw_size_per_reader)
+      if (self->reader_options.super.init_window_size < min_iw_size_per_reader)
         {
           msg_warning("WARNING: window sizing for tcp sources were changed in " VERSION_3_3
                       ", the configuration value was divided by the value of max-connections(). The result was too small, clamping to value of min_iw_size_per_reader. Ensure you have a proper log_fifo_size setting to avoid message loss.",
                       evt_tag_int("orig_log_iw_size", self->reader_options.super.init_window_size),
-                      evt_tag_int("new_log_iw_size", cfg->min_iw_size_per_reader),
-                      evt_tag_int("min_iw_size_per_reader", cfg->min_iw_size_per_reader),
-                      evt_tag_int("min_log_fifo_size", cfg->min_iw_size_per_reader * self->max_connections));
-          self->reader_options.super.init_window_size = cfg->min_iw_size_per_reader;
+                      evt_tag_int("new_log_iw_size", min_iw_size_per_reader),
+                      evt_tag_int("min_iw_size_per_reader", min_iw_size_per_reader),
+                      evt_tag_int("min_log_fifo_size", min_iw_size_per_reader * self->max_connections));
+          self->reader_options.super.init_window_size = min_iw_size_per_reader;
         }
       self->window_size_initialized = TRUE;
     }
@@ -565,6 +792,7 @@ _finalize_init(gpointer arg)
       return FALSE;
     }
 
+  _listen_fd_init(self);
   afsocket_sd_start_watches(self);
   char buf[256];
   msg_info("Accepting connections",
@@ -591,8 +819,8 @@ _sd_open_stream(AFSocketSourceDriver *self)
       if (!afsocket_sd_acquire_socket(self, &sock))
         return self->super.super.optional;
       if (sock == -1
-          && !transport_mapper_open_socket(self->transport_mapper, self->socket_options, self->bind_addr, AFSOCKET_DIR_RECV,
-                                           &sock))
+          && !transport_mapper_open_socket(self->transport_mapper, self->socket_options, self->bind_addr,
+                                           self->bind_addr, AFSOCKET_DIR_RECV, &sock))
         return self->super.super.optional;
     }
   self->fd = sock;
@@ -608,8 +836,8 @@ _sd_open_dgram(AFSocketSourceDriver *self)
       if (!afsocket_sd_acquire_socket(self, &sock))
         return self->super.super.optional;
       if (sock == -1
-          && !transport_mapper_open_socket(self->transport_mapper, self->socket_options, self->bind_addr, AFSOCKET_DIR_RECV,
-                                           &sock))
+          && !transport_mapper_open_socket(self->transport_mapper, self->socket_options, self->bind_addr,
+                                           self->bind_addr, AFSOCKET_DIR_RECV, &sock))
         return self->super.super.optional;
     }
   self->fd = -1;
@@ -691,6 +919,40 @@ afsocket_sd_save_listener(AFSocketSourceDriver *self)
     }
 }
 
+static void
+afsocket_sd_save_dynamic_window_pool(AFSocketSourceDriver *self)
+{
+  GlobalConfig *cfg = log_pipe_get_config(&self->super.super.super);
+
+  if (self->connections_kept_alive_across_reloads)
+    {
+      cfg_persist_config_add(cfg, afsocket_sd_format_dynamic_window_pool_name(self),
+                             self->dynamic_window_pool, (GDestroyNotify) dynamic_window_pool_unref, FALSE);
+    }
+  else
+    {
+      dynamic_window_pool_unref(self->dynamic_window_pool);
+    }
+
+  self->dynamic_window_pool = NULL;
+}
+
+static gboolean
+afsocket_sd_restore_dynamic_window_pool(AFSocketSourceDriver *self)
+{
+  GlobalConfig *cfg = log_pipe_get_config(&self->super.super.super);
+
+  if (!self->connections_kept_alive_across_reloads)
+    return FALSE;
+
+  DynamicWindowPool *ctr = cfg_persist_config_fetch(cfg, afsocket_sd_format_dynamic_window_pool_name(self));
+  if (ctr == NULL)
+    return FALSE;
+
+  self->dynamic_window_pool = ctr;
+  return TRUE;
+}
+
 
 gboolean
 afsocket_sd_setup_addresses_method(AFSocketSourceDriver *self)
@@ -703,11 +965,22 @@ afsocket_sd_init_method(LogPipe *s)
 {
   AFSocketSourceDriver *self = (AFSocketSourceDriver *) s;
 
-  return log_src_driver_init_method(s) &&
-         afsocket_sd_setup_transport(self) &&
-         afsocket_sd_setup_addresses(self) &&
-         afsocket_sd_restore_kept_alive_connections(self) &&
-         afsocket_sd_open_listener(self);
+  if (!log_src_driver_init_method(s))
+    return FALSE;
+
+  if (!afsocket_sd_setup_transport(self) || !afsocket_sd_setup_addresses(self))
+    return FALSE;
+
+  if (!afsocket_sd_restore_dynamic_window_pool(self))
+    {
+      if (self->dynamic_window_size != 0)
+        {
+          self->dynamic_window_pool = dynamic_window_pool_new(self->dynamic_window_size);
+          dynamic_window_pool_init(self->dynamic_window_pool);
+        }
+    }
+
+  return afsocket_sd_restore_kept_alive_connections(self) && afsocket_sd_open_listener(self);
 }
 
 gboolean
@@ -718,6 +991,9 @@ afsocket_sd_deinit_method(LogPipe *s)
   afsocket_sd_save_connections(self);
   afsocket_sd_save_listener(self);
 
+  if (self->dynamic_window_pool)
+    afsocket_sd_save_dynamic_window_pool(self);
+
   return log_src_driver_deinit_method(s);
 }
 
@@ -768,17 +1044,12 @@ afsocket_sd_init_instance(AFSocketSourceDriver *self,
   self->transport_mapper = transport_mapper;
   self->max_connections = 10;
   self->listen_backlog = 255;
+  self->dynamic_window_stats_freq = DYNAMIC_WINDOW_TIMER_MSECS;
+  self->dynamic_window_realloc_ticks = DYNAMIC_WINDOW_REALLOC_TICKS;
   self->connections_kept_alive_across_reloads = TRUE;
   log_reader_options_defaults(&self->reader_options);
   self->reader_options.super.stats_level = STATS_LEVEL1;
   self->reader_options.super.stats_source = transport_mapper->stats_source;
 
-  /* NOTE: this changes the initial window size from 100 to 1000. Reasons:
-   * Starting with syslog-ng 3.3, window-size is distributed evenly between
-   * _all_ possible connections to avoid starving.  With the defaults this
-   * means that we get a window size of 10 messages log_iw_size(100) /
-   * max_connections(10), but that is incredibly slow, thus bump this value here.
-   */
-
-  self->reader_options.super.init_window_size = 1000;
+  _init_watches(self);
 }
