@@ -21,11 +21,103 @@
  *
  */
 
+#include <math.h>
+
 typedef gboolean (*AggregateFunc)(gpointer, gint64);
 
+typedef struct _Number
+{
+  enum
+  {
+    Integer,
+    Float
+  } value_type;
+  union
+  {
+    gint64 raw_integer;
+    gdouble raw_float;
+  } value_data;
+  int precision;
+} Number;
+
+gdouble
+number_as_double(Number number)
+{
+  if (number.value_type == Float)
+    return number.value_data.raw_float;
+
+  return number.value_data.raw_integer;
+}
+
+void
+number_set_double(Number *number, double value)
+{
+  number->value_type = Float;
+  number->value_data.raw_float = value;
+  number->precision = 20;
+}
+
+gint64
+number_as_int(Number number)
+{
+  if (number.value_type == Integer)
+    return number.value_data.raw_integer;
+
+  return number.value_data.raw_float;
+}
+
+void
+number_set_int(Number *number, gint64 value)
+{
+  number->value_type = Integer;
+  number->value_data.raw_integer = value;
+  number->precision = 0;
+}
+
+gboolean
+number_is_zero(Number number)
+{
+  if (number.value_type == Integer)
+    return number.value_data.raw_integer == 0;
+
+  return fabs(number.value_data.raw_float) < DBL_EPSILON;
+}
+
+gboolean
+parse_integer_or_float(const char *str, Number *number)
+{
+  gint64 int_value;
+  if (parse_dec_number(str, &int_value))
+    {
+      number_set_int(number, int_value);
+      return TRUE;
+    }
+
+  double float_value;
+  if (parse_float(str, &float_value))
+    {
+      number_set_double(number, float_value);
+      return TRUE;
+    }
+
+  return FALSE;
+}
+
+void
+format_number(GString *result, Number n)
+{
+  if (n.value_type == Integer)
+    {
+      format_int64_padded(result, 0, ' ', 10, number_as_int(n));
+      return;
+    }
+
+  g_string_append_printf(result, "%.*f", n.precision, number_as_double(n));
+}
+
 static gboolean
 tf_num_parse(gint argc, GString *argv[],
-             const gchar *func_name, gint64 *n, gint64 *m)
+             const gchar *func_name, Number *n, Number *m)
 {
   if (argc != 2)
     {
@@ -34,7 +126,7 @@ tf_num_parse(gint argc, GString *argv[],
       return FALSE;
     }
 
-  if (!parse_dec_number(argv[0]->str, n))
+  if (!parse_integer_or_float(argv[0]->str, n))
     {
       msg_debug("Parsing failed, template function's first argument is not a number",
                 evt_tag_str("function", func_name),
@@ -42,7 +134,7 @@ tf_num_parse(gint argc, GString *argv[],
       return FALSE;
     }
 
-  if (!parse_dec_number(argv[1]->str, m))
+  if (!parse_integer_or_float(argv[1]->str, m))
     {
       msg_debug("Parsing failed, template function's second argument is not a number",
                 evt_tag_str("function", func_name),
@@ -56,7 +148,7 @@ tf_num_parse(gint argc, GString *argv[],
 static void
 tf_num_plus(LogMessage *msg, gint argc, GString *argv[], GString *result)
 {
-  gint64 n, m;
+  Number n, m, res;
 
   if (!tf_num_parse(argc, argv, "+", &n, &m))
     {
@@ -64,7 +156,16 @@ tf_num_plus(LogMessage *msg, gint argc, GString *argv[], GString *result)
       return;
     }
 
-  format_int64_padded(result, 0, ' ', 10, n + m);
+  if (n.value_type == Integer && m.value_type == Integer)
+    {
+      number_set_int(&res, number_as_int(n) + number_as_int(m));
+    }
+  else
+    {
+      number_set_double(&res, number_as_double(n) + number_as_double(m));
+    }
+
+  format_number(result, res);
 }
 
 TEMPLATE_FUNCTION_SIMPLE(tf_num_plus);
@@ -72,7 +173,7 @@ TEMPLATE_FUNCTION_SIMPLE(tf_num_plus);
 static void
 tf_num_minus(LogMessage *msg, gint argc, GString *argv[], GString *result)
 {
-  gint64 n, m;
+  Number n, m, res;
 
   if (!tf_num_parse(argc, argv, "-", &n, &m))
     {
@@ -80,7 +181,16 @@ tf_num_minus(LogMessage *msg, gint argc, GString *argv[], GString *result)
       return;
     }
 
-  format_int64_padded(result, 0, ' ', 10, n - m);
+  if (n.value_type == Integer && m.value_type == Integer)
+    {
+      number_set_int(&res, number_as_int(n) - number_as_int(m));
+    }
+  else
+    {
+      number_set_double(&res, number_as_double(n) - number_as_double(m));
+    }
+
+  format_number(result, res);
 }
 
 TEMPLATE_FUNCTION_SIMPLE(tf_num_minus);
@@ -88,7 +198,7 @@ TEMPLATE_FUNCTION_SIMPLE(tf_num_minus);
 static void
 tf_num_multi(LogMessage *msg, gint argc, GString *argv[], GString *result)
 {
-  gint64 n, m;
+  Number n, m, res;
 
   if (!tf_num_parse(argc, argv, "*", &n, &m))
     {
@@ -96,7 +206,16 @@ tf_num_multi(LogMessage *msg, gint argc, GString *argv[], GString *result)
       return;
     }
 
-  format_int64_padded(result, 0, ' ', 10, n * m);
+  if (n.value_type == Integer && m.value_type == Integer)
+    {
+      number_set_int(&res, number_as_int(n) * number_as_int(m));
+    }
+  else
+    {
+      number_set_double(&res, number_as_double(n) * number_as_double(m));
+    }
+
+  format_number(result, res);
 }
 
 TEMPLATE_FUNCTION_SIMPLE(tf_num_multi);
@@ -104,15 +223,24 @@ TEMPLATE_FUNCTION_SIMPLE(tf_num_multi);
 static void
 tf_num_div(LogMessage *msg, gint argc, GString *argv[], GString *result)
 {
-  gint64 n, m;
+  Number n, m, res;
 
-  if (!tf_num_parse(argc, argv, "/", &n, &m) || !m)
+  if (!tf_num_parse(argc, argv, "/", &n, &m) || number_is_zero(m))
     {
       g_string_append_len(result, "NaN", 3);
       return;
     }
 
-  format_int64_padded(result, 0, ' ', 10, n / m);
+  if (n.value_type == Integer && m.value_type == Integer)
+    {
+      number_set_int(&res, number_as_int(n) / number_as_int(m));
+    }
+  else
+    {
+      number_set_double(&res, number_as_double(n) / number_as_double(m));
+    }
+
+  format_number(result, res);
 }
 
 TEMPLATE_FUNCTION_SIMPLE(tf_num_div);
@@ -120,19 +248,140 @@ TEMPLATE_FUNCTION_SIMPLE(tf_num_div);
 static void
 tf_num_mod(LogMessage *msg, gint argc, GString *argv[], GString *result)
 {
-  gint64 n, m;
+  Number n, m, res;
 
-  if (!tf_num_parse(argc, argv, "%", &n, &m) || !m)
+  if (!tf_num_parse(argc, argv, "%", &n, &m) || number_is_zero(m))
     {
       g_string_append_len(result, "NaN", 3);
       return;
     }
 
-  format_uint64_padded(result, 0, ' ', 10, n % m);
+  if (n.value_type == Integer && m.value_type == Integer)
+    {
+      number_set_int(&res, number_as_int(n) % number_as_int(m));
+    }
+  else
+    {
+      number_set_double(&res, fmod(number_as_double(n), number_as_double(m)));
+    }
+
+  format_number(result, res);
 }
 
 TEMPLATE_FUNCTION_SIMPLE(tf_num_mod);
 
+static void
+tf_num_round(LogMessage *msg, gint argc, GString *argv[], GString *result)
+{
+  Number n;
+  gint64 precision = 0;
+
+  if (argc < 1 || argc > 2)
+    {
+      msg_debug("Template function requires exactly one or two arguments.",
+                evt_tag_str("function", "round"));
+      g_string_append_len(result, "NaN", 3);
+      return;
+    }
+
+  if (!parse_integer_or_float(argv[0]->str, &n))
+    {
+      msg_debug("Parsing failed, template function's first argument is not a number",
+                evt_tag_str("function", "round"),
+                evt_tag_str("arg1", argv[0]->str));
+      g_string_append_len(result, "NaN", 3);
+      return;
+    }
+
+  if (argc > 1)
+    {
+      if (!parse_dec_number(argv[1]->str, &precision))
+        {
+          msg_debug("Parsing failed, template function's second argument is not a number",
+                    evt_tag_str("function", "round"),
+                    evt_tag_str("arg2", argv[1]->str));
+          g_string_append_len(result, "NaN", 3);
+          return;
+        }
+
+      if (precision < 0 || precision > 20)
+        {
+          msg_debug("Parsing failed, precision is not in the supported range (0..20)",
+                    evt_tag_str("function", "round"),
+                    evt_tag_str("arg2", argv[1]->str));
+          g_string_append_len(result, "NaN", 3);
+          return;
+        }
+    }
+
+  double multiplier = pow(10, precision);
+  double res = round(number_as_double(n) * multiplier) / multiplier;
+  number_set_double(&n, res);
+
+  /*
+   * number_set_double() resets the precision, so assign it now.
+   */
+  n.precision = precision;
+
+  format_number(result, n);
+}
+
+TEMPLATE_FUNCTION_SIMPLE(tf_num_round);
+
+static void
+tf_num_ceil(LogMessage *msg, gint argc, GString *argv[], GString *result)
+{
+  Number n;
+
+  if (argc != 1)
+    {
+      msg_debug("Template function requires one argument.",
+                evt_tag_str("function", "ceil"));
+      g_string_append_len(result, "NaN", 3);
+      return;
+    }
+
+  if (!parse_integer_or_float(argv[0]->str, &n))
+    {
+      msg_debug("Parsing failed, template function's first argument is not a number",
+                evt_tag_str("function", "ceil"),
+                evt_tag_str("arg1", argv[0]->str));
+      g_string_append_len(result, "NaN", 3);
+      return;
+    }
+
+  format_int64_padded(result, 0, ' ', 10, ceil(number_as_double(n)));
+}
+
+TEMPLATE_FUNCTION_SIMPLE(tf_num_ceil);
+
+static void
+tf_num_floor(LogMessage *msg, gint argc, GString *argv[], GString *result)
+{
+  Number n;
+
+  if (argc != 1)
+    {
+      msg_debug("Template function requires one argument.",
+                evt_tag_str("function", "floor"));
+      g_string_append_len(result, "NaN", 3);
+      return;
+    }
+
+  if (!parse_integer_or_float(argv[0]->str, &n))
+    {
+      msg_debug("Parsing failed, template function's first argument is not a number",
+                evt_tag_str("function", "floor"),
+                evt_tag_str("arg1", argv[0]->str));
+      g_string_append_len(result, "NaN", 3);
+      return;
+    }
+
+  format_int64_padded(result, 0, ' ', 10, floor(number_as_double(n)));
+}
+
+TEMPLATE_FUNCTION_SIMPLE(tf_num_floor);
+
 static gboolean
 _tf_num_parse_arg_with_message(const TFSimpleFuncState *state,
                                LogMessage *message,
