@@ -29,5 +29,7 @@
 gint pdb_file_detect_version(const gchar *pdbfile, GError **error);
 gboolean pdb_file_validate(const gchar *filename, GError **error);
 gboolean pdb_file_validate_in_tests(const gchar *filename, GError **error);
+GPtrArray *pdb_get_filenames(const gchar *dir_path, gboolean recursive, gchar *pattern, GError **error);
+void pdb_sort_filenames(GPtrArray *filenames);
 
 #endif
