@@ -48,7 +48,7 @@ _get_length(LogQueue *s)
   LogQueueDisk *self = (LogQueueDisk *) s;
   gint64 qdisk_length = 0;
 
-  if (qdisk_initialized(self->qdisk) && self->get_length)
+  if (qdisk_started(self->qdisk) && self->get_length)
     {
       qdisk_length = self->get_length(self);
     }
@@ -165,7 +165,7 @@ log_queue_disk_save_queue(LogQueue *s, gboolean *persistent)
 {
   LogQueueDisk *self = (LogQueueDisk *) s;
 
-  if (!qdisk_initialized(self->qdisk))
+  if (!qdisk_started(self->qdisk))
     {
       *persistent = FALSE;
       return TRUE;
@@ -182,7 +182,7 @@ log_queue_disk_load_queue(LogQueue *s, const gchar *filename)
   LogQueueDisk *self = (LogQueueDisk *) s;
 
   /* qdisk portion is not yet started when this happens */
-  g_assert(!qdisk_initialized(self->qdisk));
+  g_assert(!qdisk_started(self->qdisk));
 
   if (self->load_queue)
     return self->load_queue(self, filename);
@@ -213,7 +213,7 @@ _free(LogQueue *s)
   if (self->free_fn)
     self->free_fn(self);
 
-  qdisk_deinit(self->qdisk);
+  qdisk_stop(self->qdisk);
   qdisk_free(self->qdisk);
 
   log_queue_free_method(s);
@@ -227,7 +227,7 @@ _pop_disk(LogQueueDisk *self, LogMessage **msg)
 
   *msg = NULL;
 
-  if (!qdisk_initialized(self->qdisk))
+  if (!qdisk_started(self->qdisk))
     return FALSE;
 
   serialized = g_string_sized_new(64);
@@ -247,7 +247,8 @@ _pop_disk(LogQueueDisk *self, LogMessage **msg)
       log_msg_unref(*msg);
       *msg = NULL;
       msg_error("Can't read correct message from disk-queue file",
-                evt_tag_str("filename", qdisk_get_filename(self->qdisk)));
+                evt_tag_str("filename", qdisk_get_filename(self->qdisk)),
+                evt_tag_long("read_position", qdisk_get_reader_head(self->qdisk)));
       return TRUE;
     }
 
@@ -288,7 +289,7 @@ _write_message(LogQueueDisk *self, LogMessage *msg)
   GString *serialized;
   SerializeArchive *sa;
   gboolean consumed = FALSE;
-  if (qdisk_initialized(self->qdisk) && qdisk_is_space_avail(self->qdisk, 64))
+  if (qdisk_started(self->qdisk) && qdisk_is_space_avail(self->qdisk, 64))
     {
       serialized = g_string_sized_new(64);
       sa = serialize_string_archive_new(serialized);
@@ -304,13 +305,17 @@ static void
 _restart_diskq(LogQueueDisk *self)
 {
   gchar *filename = g_strdup(qdisk_get_filename(self->qdisk));
-  gchar *new_file = NULL;
   DiskQueueOptions *options = qdisk_get_options(self->qdisk);
 
-  qdisk_deinit(self->qdisk);
+  qdisk_stop(self->qdisk);
 
-  new_file = g_strdup_printf("%s.corrupted", filename);
-  rename(filename, new_file);
+  gchar *new_file = g_strdup_printf("%s.corrupted", filename);
+  if (rename(filename, new_file) < 0)
+    {
+      msg_error("Moving corrupt disk-queue failed",
+                evt_tag_str(EVT_TAG_FILENAME, filename),
+                evt_tag_error(EVT_TAG_OSERROR));
+    }
   g_free(new_file);
 
   if (self->restart)
