@@ -155,6 +155,37 @@ _add_header(struct curl_slist *curl_headers, const gchar *header, const gchar *v
   return curl_slist_append(curl_headers, buffer->str);
 }
 
+static struct curl_slist *
+_append_auth_header(struct curl_slist *list, HTTPDestinationDriver *owner)
+{
+  const gchar *auth_header_str = http_auth_header_get_as_string(owner->auth_header);
+
+  if (!auth_header_str)
+    {
+      if (!http_dd_auth_header_renew(&owner->super.super.super))
+        {
+          msg_warning("WARNING: failed to renew auth header",
+                      evt_tag_str("driver", owner->super.super.super.id),
+                      log_pipe_location_tag(&owner->super.super.super.super));
+          return list;
+        }
+      auth_header_str = http_auth_header_get_as_string(owner->auth_header);
+    }
+
+  if (auth_header_str)
+    {
+      list = curl_slist_append(list, auth_header_str);
+    }
+  else
+    {
+      msg_warning("WARNING: auth-header() returned NULL-value",
+                  evt_tag_str("driver", owner->super.super.super.id),
+                  log_pipe_location_tag(&owner->super.super.super.super));
+    }
+
+  return list;
+}
+
 static struct curl_slist *
 _format_request_headers(HTTPDestinationWorker *self, LogMessage *msg)
 {
@@ -189,6 +220,9 @@ _format_request_headers(HTTPDestinationWorker *self, LogMessage *msg)
   for (l = owner->headers; l; l = l->next)
     headers = curl_slist_append(headers, l->data);
 
+  if (owner->auth_header)
+    headers = _append_auth_header(headers, owner);
+
   return headers;
 }
 
@@ -212,51 +246,108 @@ _add_message_to_batch(HTTPDestinationWorker *self, LogMessage *msg)
     }
 }
 
-worker_insert_result_t
-map_http_status_to_worker_status(HTTPDestinationWorker *self, const gchar *url, glong http_code)
+static gboolean
+_find_http_code_in_list(glong http_code, glong list[])
+{
+  for (gint i = 0; list[i] != -1; i++)
+    if (list[i] == http_code)
+      return TRUE;
+  return FALSE;
+}
+
+static LogThreadedResult
+_default_1XX(HTTPDestinationWorker *self, const gchar *url, glong http_code)
 {
   HTTPDestinationDriver *owner = (HTTPDestinationDriver *) self->super.owner;
-  worker_insert_result_t retval = WORKER_INSERT_RESULT_ERROR;
+  msg_error("Server returned with a 1XX (continuation) status code, which was not handled by curl. ",
+            evt_tag_str("url", url),
+            evt_tag_int("status_code", http_code),
+            evt_tag_str("driver", owner->super.super.super.id),
+            log_pipe_location_tag(&owner->super.super.super.super));
+
+  static glong errors[] = {102, 103, -1};
+  if (_find_http_code_in_list(http_code, errors))
+    return LTR_ERROR;
+
+  return LTR_NOT_CONNECTED;
+}
+
+static LogThreadedResult
+_default_3XX(HTTPDestinationWorker *self, const gchar *url, glong http_code)
+{
+  HTTPDestinationDriver *owner = (HTTPDestinationDriver *) self->super.owner;
+  msg_notice("Server returned with a 3XX (redirect) status code. "
+             "Either accept-redirect() is set to no, or this status code is unknown.",
+             evt_tag_str("url", url),
+             evt_tag_int("status_code", http_code),
+             evt_tag_str("driver", owner->super.super.super.id),
+             log_pipe_location_tag(&owner->super.super.super.super));
+  if (http_code == 304)
+    return LTR_ERROR;
+
+  return LTR_NOT_CONNECTED;
+}
+
+static LogThreadedResult
+_default_4XX(HTTPDestinationWorker *self, const gchar *url, glong http_code)
+{
+  HTTPDestinationDriver *owner = (HTTPDestinationDriver *) self->super.owner;
+  msg_notice("Server returned with a 4XX (client errors) status code, which means we are not "
+             "authorized or the URL is not found.",
+             evt_tag_str("url", url),
+             evt_tag_int("status_code", http_code),
+             evt_tag_str("driver", owner->super.super.super.id),
+             log_pipe_location_tag(&owner->super.super.super.super));
+
+  static glong errors[] = {428, -1};
+  if (_find_http_code_in_list(http_code, errors))
+    return LTR_ERROR;
+
+  static glong drops[] = {410, 416, 422, 424, 425, 451, -1};
+  if (_find_http_code_in_list(http_code, drops))
+    return LTR_DROP;
+
+  return LTR_NOT_CONNECTED;
+}
+
+static LogThreadedResult
+_default_5XX(HTTPDestinationWorker *self, const gchar *url, glong http_code)
+{
+  HTTPDestinationDriver *owner = (HTTPDestinationDriver *) self->super.owner;
+  msg_notice("Server returned with a 5XX (server errors) status code, which indicates server failure.",
+             evt_tag_str("url", url),
+             evt_tag_int("status_code", http_code),
+             evt_tag_str("driver", owner->super.super.super.id),
+             log_pipe_location_tag(&owner->super.super.super.super));
+  if (http_code == 508)
+    return LTR_DROP;
+
+  static glong errors[] = {504, -1};
+  if (_find_http_code_in_list(http_code, errors))
+    return LTR_ERROR;
+
+  return LTR_NOT_CONNECTED;
+}
+
+LogThreadedResult
+default_map_http_status_to_worker_status(HTTPDestinationWorker *self, const gchar *url, glong http_code)
+{
+  HTTPDestinationDriver *owner = (HTTPDestinationDriver *) self->super.owner;
+  LogThreadedResult retval = LTR_ERROR;
 
   switch (http_code/100)
     {
     case 1:
-      msg_error("Server returned with a 1XX (continuation) status code, which was not handled by curl. "
-                "Trying again",
-                evt_tag_str("url", owner->url),
-                evt_tag_int("status_code", http_code),
-                evt_tag_str("driver", owner->super.super.super.id),
-                log_pipe_location_tag(&owner->super.super.super.super));
-      break;
+      return _default_1XX(self, url, http_code);
     case 2:
       /* everything is dandy */
-      retval = WORKER_INSERT_RESULT_SUCCESS;
-      break;
+      return  LTR_SUCCESS;
     case 3:
-      msg_notice("Server returned with a 3XX (redirect) status code, which was not handled by curl. "
-                 "Either accept-redirect() is set to no, or this status code is unknown. Trying again",
-                 evt_tag_str("url", url),
-                 evt_tag_int("status_code", http_code),
-                 evt_tag_str("driver", owner->super.super.super.id),
-                 log_pipe_location_tag(&owner->super.super.super.super));
-      break;
+      return _default_3XX(self, url, http_code);
     case 4:
-      msg_notice("Server returned with a 4XX (client errors) status code, which means we are not "
-                 "authorized or the URL is not found.",
-                 evt_tag_str("url", url),
-                 evt_tag_int("status_code", http_code),
-                 evt_tag_str("driver", owner->super.super.super.id),
-                 log_pipe_location_tag(&owner->super.super.super.super));
-      retval = WORKER_INSERT_RESULT_DROP;
-      break;
+      return _default_4XX(self, url, http_code);
     case 5:
-      msg_notice("Server returned with a 5XX (server errors) status code, which indicates server failure. "
-                 "Trying again",
-                 evt_tag_str("url", owner->url),
-                 evt_tag_int("status_code", http_code),
-                 evt_tag_str("driver", owner->super.super.super.id),
-                 log_pipe_location_tag(&owner->super.super.super.super));
-      break;
+      return _default_5XX(self, url, http_code);
     default:
       msg_error("Unknown HTTP response code",
                 evt_tag_str("url", url),
@@ -289,19 +380,95 @@ _finish_request_body(HTTPDestinationWorker *self)
     g_string_append_len(self->request_body, owner->body_suffix->str, owner->body_suffix->len);
 }
 
-static worker_insert_result_t
-_flush_on_target(HTTPDestinationWorker *self, HTTPLoadBalancerTarget *target)
+static void
+_debug_response_info(HTTPDestinationWorker *self, HTTPLoadBalancerTarget *target, glong http_code)
+{
+  HTTPDestinationDriver *owner = (HTTPDestinationDriver *) self->super.owner;
+
+  gdouble total_time = 0;
+  glong redirect_count = 0;
+
+  curl_easy_getinfo(self->curl, CURLINFO_TOTAL_TIME, &total_time);
+  curl_easy_getinfo(self->curl, CURLINFO_REDIRECT_COUNT, &redirect_count);
+  msg_debug("curl: HTTP response received",
+            evt_tag_str("url", target->url),
+            evt_tag_int("status_code", http_code),
+            evt_tag_int("body_size", self->request_body->len),
+            evt_tag_int("batch_size", self->super.batch_size),
+            evt_tag_int("redirected", redirect_count != 0),
+            evt_tag_printf("total_time", "%.3f", total_time),
+            evt_tag_int("worker_index", self->super.worker_index),
+            evt_tag_str("driver", owner->super.super.super.id),
+            log_pipe_location_tag(&owner->super.super.super.super));
+}
+
+static LogThreadedResult
+_custom_map_http_result(HTTPDestinationWorker *self, const gchar *url, HttpResponseHandler *response_handler)
+{
+  HttpResult result = response_handler->action(response_handler->user_data);
+  g_assert(result < HTTP_RESULT_MAX);
+  HTTPDestinationDriver *owner = (HTTPDestinationDriver *) self->super.owner;
+  glong http_code = response_handler->status_code;
+
+  switch (result)
+    {
+    case HTTP_RESULT_SUCCESS:
+      msg_debug("http: handled by response_action",
+                evt_tag_str("action", "success"),
+                evt_tag_str("url", url),
+                evt_tag_int("status_code", http_code),
+                evt_tag_str("driver", owner->super.super.super.id),
+                log_pipe_location_tag(&owner->super.super.super.super));
+      return LTR_SUCCESS;
+
+    case HTTP_RESULT_RETRY:
+      msg_notice("http: handled by response_action",
+                 evt_tag_str("action", "retry"),
+                 evt_tag_str("url", url),
+                 evt_tag_int("status_code", http_code),
+                 evt_tag_str("driver", owner->super.super.super.id),
+                 log_pipe_location_tag(&owner->super.super.super.super));
+      return LTR_ERROR;
+
+    case HTTP_RESULT_DROP:
+      msg_notice("http: handled by response_action",
+                 evt_tag_str("action", "drop"),
+                 evt_tag_str("url", url),
+                 evt_tag_int("status_code", http_code),
+                 evt_tag_str("driver", owner->super.super.super.id),
+                 log_pipe_location_tag(&owner->super.super.super.super));
+      return LTR_DROP;
+
+    case HTTP_RESULT_DISCONNECT:
+      msg_notice("http: handled by response_action",
+                 evt_tag_str("action", "disconnect"),
+                 evt_tag_str("url", url),
+                 evt_tag_int("status_code", http_code),
+                 evt_tag_str("driver", owner->super.super.super.id),
+                 log_pipe_location_tag(&owner->super.super.super.super));
+      return LTR_NOT_CONNECTED;
+
+    default:
+      g_assert_not_reached();
+    };
+
+  return LTR_MAX;
+}
+
+static gboolean
+_curl_perform_request(HTTPDestinationWorker *self, HTTPLoadBalancerTarget *target)
 {
   HTTPDestinationDriver *owner = (HTTPDestinationDriver *) self->super.owner;
-  CURLcode ret;
 
   msg_trace("Sending HTTP request",
             evt_tag_str("url", target->url));
+
   curl_easy_setopt(self->curl, CURLOPT_URL, target->url);
   curl_easy_setopt(self->curl, CURLOPT_HTTPHEADER, self->request_headers);
   curl_easy_setopt(self->curl, CURLOPT_POSTFIELDS, self->request_body->str);
 
-  if ((ret = curl_easy_perform(self->curl)) != CURLE_OK)
+  CURLcode ret = curl_easy_perform(self->curl);
+  if (ret != CURLE_OK)
     {
       msg_error("curl: error sending HTTP request",
                 evt_tag_str("url", target->url),
@@ -309,13 +476,19 @@ _flush_on_target(HTTPDestinationWorker *self, HTTPLoadBalancerTarget *target)
                 evt_tag_int("worker_index", self->super.worker_index),
                 evt_tag_str("driver", owner->super.super.super.id),
                 log_pipe_location_tag(&owner->super.super.super.super));
-      return WORKER_INSERT_RESULT_NOT_CONNECTED;
+      return FALSE;
     }
 
-  glong http_code = 0;
+  return TRUE;
+}
 
-  CURLcode code = curl_easy_getinfo(self->curl, CURLINFO_RESPONSE_CODE, &http_code);
-  if (code != CURLE_OK)
+static gboolean
+_curl_get_status_code(HTTPDestinationWorker *self, HTTPLoadBalancerTarget *target, glong *http_code)
+{
+  HTTPDestinationDriver *owner = (HTTPDestinationDriver *) self->super.owner;
+  CURLcode ret = curl_easy_getinfo(self->curl, CURLINFO_RESPONSE_CODE, http_code);
+
+  if (ret != CURLE_OK)
     {
       msg_error("curl: error querying response code",
                 evt_tag_str("url", target->url),
@@ -323,45 +496,83 @@ _flush_on_target(HTTPDestinationWorker *self, HTTPLoadBalancerTarget *target)
                 evt_tag_int("worker_index", self->super.worker_index),
                 evt_tag_str("driver", owner->super.super.super.id),
                 log_pipe_location_tag(&owner->super.super.super.super));
-      return WORKER_INSERT_RESULT_NOT_CONNECTED;
+      return FALSE;
     }
 
+  return TRUE;
+}
+
+static LogThreadedResult
+_renew_header(HTTPDestinationDriver *self)
+{
+  if (!http_dd_auth_header_renew(&self->super.super.super))
+    return LTR_NOT_CONNECTED;
+  return LTR_RETRY;
+}
+
+static LogThreadedResult
+_try_to_custom_map_http_status_to_worker_status(HTTPDestinationWorker *self, const gchar *url, glong http_code)
+{
+  HttpResponseHandler *response_handler = NULL;
+
+  HTTPDestinationDriver *owner = (HTTPDestinationDriver *) self->super.owner;
+  if ((response_handler = http_response_handlers_lookup(owner->response_handlers, http_code)))
+    return _custom_map_http_result(self, url, response_handler);
+
+  return LTR_MAX;
+}
+
+static LogThreadedResult
+_map_http_status_code(HTTPDestinationWorker *self, const gchar *url, glong http_code)
+{
+  LogThreadedResult result = _try_to_custom_map_http_status_to_worker_status(self, url, http_code);
+
+  if (result != LTR_MAX)
+    return result;
+
+  return default_map_http_status_to_worker_status(self, url, http_code);
+}
+
+static LogThreadedResult
+_flush_on_target(HTTPDestinationWorker *self, HTTPLoadBalancerTarget *target)
+{
+  HTTPDestinationDriver *owner = (HTTPDestinationDriver *) self->super.owner;
+
+  if (!_curl_perform_request(self, target))
+    return LTR_NOT_CONNECTED;
+
+  glong http_code = 0;
+
+  if (!_curl_get_status_code(self, target, &http_code))
+    return LTR_NOT_CONNECTED;
+
   if (debug_flag)
-    {
-      gdouble total_time = 0;
-      glong redirect_count = 0;
+    _debug_response_info(self, target, http_code);
 
-      curl_easy_getinfo(self->curl, CURLINFO_TOTAL_TIME, &total_time);
-      curl_easy_getinfo(self->curl, CURLINFO_REDIRECT_COUNT, &redirect_count);
-      msg_debug("curl: HTTP response received",
-                evt_tag_str("url", target->url),
-                evt_tag_int("status_code", http_code),
-                evt_tag_int("body_size", self->request_body->len),
-                evt_tag_int("batch_size", self->super.batch_size),
-                evt_tag_int("redirected", redirect_count != 0),
-                evt_tag_printf("total_time", "%.3f", total_time),
-                evt_tag_int("worker_index", self->super.worker_index),
-                evt_tag_str("driver", owner->super.super.super.id),
-                log_pipe_location_tag(&owner->super.super.super.super));
-    }
-  return map_http_status_to_worker_status(self, target->url, http_code);
+  if (http_code == 401 && owner->auth_header)
+    return _renew_header(owner);
+
+  return _map_http_status_code(self, target->url, http_code);
 }
 
 /* we flush the accumulated data if
  *   1) we reach batch_size,
  *   2) the message queue becomes empty
  */
-static worker_insert_result_t
-_flush(LogThreadedDestWorker *s)
+static LogThreadedResult
+_flush(LogThreadedDestWorker *s, LogThreadedFlushMode mode)
 {
   HTTPDestinationWorker *self = (HTTPDestinationWorker *) s;
   HTTPDestinationDriver *owner = (HTTPDestinationDriver *) s->owner;
   HTTPLoadBalancerTarget *target, *alt_target = NULL;
-  worker_insert_result_t retval = WORKER_INSERT_RESULT_NOT_CONNECTED;
+  LogThreadedResult retval = LTR_NOT_CONNECTED;
   gint retry_attempts = owner->load_balancer->num_targets;
 
   if (self->super.batch_size == 0)
-    return WORKER_INSERT_RESULT_SUCCESS;
+    return LTR_SUCCESS;
+
+  if (mode == LTF_FLUSH_EXPEDITE)
+    return LTR_RETRY;
 
   _finish_request_body(self);
 
@@ -370,7 +581,7 @@ _flush(LogThreadedDestWorker *s)
   while (--retry_attempts >= 0)
     {
       retval = _flush_on_target(self, target);
-      if (retval == WORKER_INSERT_RESULT_SUCCESS)
+      if (retval == LTR_SUCCESS)
         {
           http_load_balancer_set_target_successful(owner->load_balancer, target);
           break;
@@ -408,12 +619,11 @@ _should_initiate_flush(HTTPDestinationWorker *self)
 {
   HTTPDestinationDriver *owner = (HTTPDestinationDriver *) self->super.owner;
 
-  return (owner->batch_bytes && self->request_body->len + owner->body_suffix->len >= owner->batch_bytes) ||
-         (owner->super.batch_lines && self->super.batch_size >= owner->super.batch_lines);
+  return (owner->batch_bytes && self->request_body->len + owner->body_suffix->len >= owner->batch_bytes);
 
 }
 
-static worker_insert_result_t
+static LogThreadedResult
 _insert_batched(LogThreadedDestWorker *s, LogMessage *msg)
 {
   HTTPDestinationWorker *self = (HTTPDestinationWorker *) s;
@@ -425,19 +635,19 @@ _insert_batched(LogThreadedDestWorker *s, LogMessage *msg)
 
   if (_should_initiate_flush(self))
     {
-      return log_threaded_dest_worker_flush(&self->super);
+      return log_threaded_dest_worker_flush(&self->super, LTF_FLUSH_NORMAL);
     }
-  return WORKER_INSERT_RESULT_QUEUED;
+  return LTR_QUEUED;
 }
 
-static worker_insert_result_t
+static LogThreadedResult
 _insert_single(LogThreadedDestWorker *s, LogMessage *msg)
 {
   HTTPDestinationWorker *self = (HTTPDestinationWorker *) s;
 
   self->request_headers = _format_request_headers(self, msg);
   _add_message_to_batch(self, msg);
-  return _flush(&self->super);
+  return log_threaded_dest_worker_flush(&self->super, LTF_FLUSH_NORMAL);
 }
 
 static gboolean
