@@ -29,7 +29,7 @@
 #include "python-logtemplate-options.h"
 #include "python-integerpointer.h"
 #include "python-helpers.h"
-#include "logthrdestdrv.h"
+#include "logthrdest/logthrdestdrv.h"
 #include "stats/stats.h"
 #include "string-list.h"
 #include "str-utils.h"
@@ -51,9 +51,12 @@ typedef struct
     PyObject *class;
     PyObject *instance;
     PyObject *is_opened;
+    PyObject *open;
     PyObject *send;
+    PyObject *flush;
     PyObject *log_template_options;
     PyObject *seqnum;
+    GPtrArray *_refs_to_clean;
   } py;
 } PythonDestDriver;
 
@@ -157,7 +160,7 @@ _dd_py_invoke_bool_method_by_name_with_args(PythonDestDriver *self, const gchar
                                                   self->super.super.super.id);
 }
 
-static gboolean
+static gboolean G_GNUC_UNUSED
 _dd_py_invoke_bool_method_by_name(PythonDestDriver *self, const gchar *method_name)
 {
   return _py_invoke_bool_method_by_name_with_args(self->py.instance, method_name, NULL, self->class,
@@ -176,7 +179,36 @@ _py_invoke_is_opened(PythonDestDriver *self)
 static gboolean
 _py_invoke_open(PythonDestDriver *self)
 {
-  return _dd_py_invoke_bool_method_by_name(self, "open");
+  if (!self->py.open)
+    return TRUE;
+
+  PyObject *ret;
+  gboolean result = FALSE;
+
+  ret = _py_invoke_function(self->py.open, NULL, self->class, self->super.super.super.id);
+  if (ret)
+    {
+      if (ret == Py_None)
+        {
+          msg_warning_once("Since " VERSION_3_25 ", the return value of open method in python destination "
+                           "is used as success/failure indicator. Please use return True or return False explicitely",
+                           evt_tag_str("class", self->class));
+          result = TRUE;
+        }
+      else
+        result = PyObject_IsTrue(ret);
+    }
+  Py_XDECREF(ret);
+
+  if (self->py.is_opened)
+    {
+      if (!result)
+        return FALSE;
+
+      return _py_invoke_is_opened(self);
+    }
+
+  return result;
 }
 
 static void
@@ -185,10 +217,71 @@ _py_invoke_close(PythonDestDriver *self)
   _dd_py_invoke_void_method_by_name(self, "close");
 }
 
-static gboolean
+static LogThreadedResult
+_as_int(PyObject *obj)
+{
+  int result = pyobject_as_int(obj);
+  if (result == -1 && PyErr_Occurred())
+    {
+      gchar buf[256];
+      msg_error("Error converting PyObject to int. Retrying message later",
+                evt_tag_str("exception", _py_format_exception_text(buf, sizeof(buf))));
+      _py_finish_exception_handling();
+      return LTR_ERROR;
+    }
+
+  if (result < 0 || result >= LTR_MAX)
+    {
+      msg_error("Python: worker insert result out of range. Retrying message later",
+                evt_tag_int("result", result));
+      return LTR_ERROR;
+    }
+
+  return result;
+}
+
+static LogThreadedResult
+_as_bool(PyObject *obj)
+{
+  return PyObject_IsTrue(obj) ? LTR_SUCCESS : LTR_ERROR;
+}
+
+static LogThreadedResult
+pyobject_to_worker_insert_result(PyObject *obj)
+{
+  if (PyBool_Check(obj))
+    return _as_bool(obj);
+  else
+    return _as_int(obj);
+}
+
+static LogThreadedResult
+_py_invoke_flush(PythonDestDriver *self)
+{
+  if (!self->py.flush)
+    return LTR_SUCCESS;
+
+  PyObject *ret = _py_invoke_function(self->py.flush, NULL, self->class, self->super.super.super.id);
+  if (!ret)
+    return LTR_ERROR;
+
+  LogThreadedResult result = pyobject_to_worker_insert_result(ret);
+  Py_XDECREF(ret);
+  return result;
+}
+
+static LogThreadedResult
 _py_invoke_send(PythonDestDriver *self, PyObject *dict)
 {
-  return _dd_py_invoke_bool_function(self, self->py.send, dict);
+  PyObject *ret;
+  ret = _py_invoke_function(self->py.send, dict, self->class, self->super.super.super.id);
+
+  if (!ret)
+    return LTR_ERROR;
+
+  LogThreadedResult result = pyobject_to_worker_insert_result(ret);
+  Py_XDECREF(ret);
+  return result;
 }
 
 static gboolean
@@ -203,9 +296,40 @@ _py_invoke_deinit(PythonDestDriver *self)
   _dd_py_invoke_void_method_by_name(self, "deinit");
 }
 
+static void
+_py_clear(PyObject *self)
+{
+  Py_CLEAR(self);
+}
+
+#define _inject_worker_insert_result(self, value) \
+  _inject_const(self, #value, LTR_ ## value)
+
+static void
+_inject_const(PythonDestDriver *self, const gchar *field_name, gint value)
+{
+  PyObject *pyint = int_as_pyobject(value);
+  PyObject_SetAttrString(self->py.class, field_name, pyint);
+  g_ptr_array_add(self->py._refs_to_clean, pyint);
+};
+
+static void
+_inject_worker_insert_result_consts(PythonDestDriver *self)
+{
+  _inject_worker_insert_result(self, DROP);
+  _inject_worker_insert_result(self, ERROR);
+  _inject_worker_insert_result(self, SUCCESS);
+  _inject_worker_insert_result(self, QUEUED);
+  _inject_worker_insert_result(self, NOT_CONNECTED);
+  _inject_worker_insert_result(self, RETRY);
+  _inject_worker_insert_result(self, MAX);
+};
+
 static gboolean
 _py_init_bindings(PythonDestDriver *self)
 {
+  self->py._refs_to_clean = g_ptr_array_new_with_free_func((GDestroyNotify)_py_clear);
+
   self->py.class = _py_resolve_qualified_name(self->class);
   if (!self->py.class)
     {
@@ -219,6 +343,8 @@ _py_init_bindings(PythonDestDriver *self)
       return FALSE;
     }
 
+  _inject_worker_insert_result_consts(self);
+
   self->py.log_template_options = py_log_template_options_new(&self->template_options);
   PyObject_SetAttrString(self->py.class, "template_options", self->py.log_template_options);
   self->py.seqnum = py_integer_pointer_new(&self->super.worker.instance.seq_num);
@@ -240,25 +366,33 @@ _py_init_bindings(PythonDestDriver *self)
 
   /* these are fast paths, store references to be faster */
   self->py.is_opened = _py_get_attr_or_null(self->py.instance, "is_opened");
+  self->py.open = _py_get_attr_or_null(self->py.instance, "open");
+  self->py.flush = _py_get_attr_or_null(self->py.instance, "flush");
   self->py.send = _py_get_attr_or_null(self->py.instance, "send");
   if (!self->py.send)
     {
       msg_error("Error initializing Python destination, class does not have a send() method",
                 evt_tag_str("driver", self->super.super.super.id),
                 evt_tag_str("class", self->class));
+      return FALSE;
     }
-  return self->py.send != NULL;
+
+  g_ptr_array_add(self->py._refs_to_clean, self->py.class);
+  g_ptr_array_add(self->py._refs_to_clean, self->py.instance);
+  g_ptr_array_add(self->py._refs_to_clean, self->py.is_opened);
+  g_ptr_array_add(self->py._refs_to_clean, self->py.open);
+  g_ptr_array_add(self->py._refs_to_clean, self->py.flush);
+  g_ptr_array_add(self->py._refs_to_clean, self->py.send);
+  g_ptr_array_add(self->py._refs_to_clean, self->py.log_template_options);
+  g_ptr_array_add(self->py._refs_to_clean, self->py.seqnum);
+
+  return TRUE;
 }
 
 static void
 _py_free_bindings(PythonDestDriver *self)
 {
-  Py_CLEAR(self->py.class);
-  Py_CLEAR(self->py.instance);
-  Py_CLEAR(self->py.is_opened);
-  Py_CLEAR(self->py.send);
-  Py_CLEAR(self->py.log_template_options);
-  Py_CLEAR(self->py.seqnum);
+  g_ptr_array_free(self->py._refs_to_clean, TRUE);
 }
 
 static gboolean
@@ -303,21 +437,20 @@ _py_construct_message(PythonDestDriver *self, LogMessage *msg, PyObject **msg_ob
 }
 
 
-static worker_insert_result_t
+static LogThreadedResult
 python_dd_insert(LogThreadedDestDriver *d, LogMessage *msg)
 {
   PythonDestDriver *self = (PythonDestDriver *)d;
-  worker_insert_result_t result = WORKER_INSERT_RESULT_ERROR;
+  LogThreadedResult result = LTR_ERROR;
   PyObject *msg_object;
   PyGILState_STATE gstate;
 
   gstate = PyGILState_Ensure();
-  if (!_py_invoke_is_opened(self))
+  if (self->py.is_opened && !_py_invoke_is_opened(self))
     {
-      _py_invoke_open(self);
-      if (!_py_invoke_is_opened(self))
+      if (!_py_invoke_open(self))
         {
-          result = WORKER_INSERT_RESULT_NOT_CONNECTED;
+          result = LTR_NOT_CONNECTED;
           goto exit;
         }
     }
@@ -325,17 +458,7 @@ python_dd_insert(LogThreadedDestDriver *d, LogMessage *msg)
   if (!_py_construct_message(self, msg, &msg_object))
     goto exit;
 
-  if (_py_invoke_send(self, msg_object))
-    {
-      result = WORKER_INSERT_RESULT_SUCCESS;
-    }
-  else
-    {
-      msg_error("Python send() method returned failure, suspending destination for time_reopen()",
-                evt_tag_str("driver", self->super.super.super.id),
-                evt_tag_str("class", self->class),
-                evt_tag_int("time_reopen", self->super.time_reopen));
-    }
+  result =_py_invoke_send(self, msg_object);
   Py_DECREF(msg_object);
 
 exit:
@@ -343,35 +466,54 @@ exit:
   return result;
 }
 
-static void
+static gboolean
 python_dd_open(PythonDestDriver *self)
 {
   PyGILState_STATE gstate;
 
   gstate = PyGILState_Ensure();
-  if (!_py_invoke_is_opened(self))
-    _py_invoke_open(self);
+  gboolean retval = _py_invoke_open(self);
 
   PyGILState_Release(gstate);
+  return retval;
 }
 
+static LogThreadedResult
+python_dd_flush(LogThreadedDestDriver *s)
+{
+  PythonDestDriver *self = (PythonDestDriver *)s;
+  PyGILState_STATE gstate;
+
+  gstate = PyGILState_Ensure();
+  LogThreadedResult result = _py_invoke_flush(self);
+  PyGILState_Release(gstate);
+  return result;
+};
+
 static void
 python_dd_close(PythonDestDriver *self)
 {
   PyGILState_STATE gstate;
 
   gstate = PyGILState_Ensure();
-  if (_py_invoke_is_opened(self))
-    _py_invoke_close(self);
+  if (self->py.is_opened)
+    {
+      if (_py_invoke_is_opened(self))
+        _py_invoke_close(self);
+    }
+  else
+    {
+      _py_invoke_close(self);
+    }
   PyGILState_Release(gstate);
 }
 
-static void
-python_dd_worker_init(LogThreadedDestDriver *d)
+static gboolean
+python_dd_connect(LogThreadedDestDriver *d)
 {
-  PythonDestDriver *self = (PythonDestDriver *)d;
+  PythonDestDriver *self = (PythonDestDriver *) d;
 
-  python_dd_open(self);
+  return python_dd_open(self);
 }
 
 static void
@@ -472,12 +614,13 @@ python_dd_new(GlobalConfig *cfg)
   self->super.super.super.super.free_fn = python_dd_free;
   self->super.super.super.super.generate_persist_name = python_dd_format_persist_name;
 
-  self->super.worker.thread_init = python_dd_worker_init;
+  self->super.worker.connect = python_dd_connect;
   self->super.worker.disconnect = python_dd_disconnect;
   self->super.worker.insert = python_dd_insert;
+  self->super.worker.flush = python_dd_flush;
 
   self->super.format_stats_instance = python_dd_format_stats_instance;
-  self->super.stats_source = SCS_PYTHON;
+  self->super.stats_source = stats_register_type("python");
 
   self->options = g_hash_table_new_full(g_str_hash, g_str_equal, g_free, g_free);
 
