@@ -45,7 +45,7 @@ _py_get_callable_name(PyObject *callable, gchar *buf, gsize buf_len)
 }
 
 void
-_py_log_python_traceback_to_stderr_in_debug_mode(void)
+_py_log_python_traceback_to_stderr(void)
 {
   PyObject *traceback_module = NULL;
   PyObject *print_exception = NULL;
@@ -53,6 +53,8 @@ _py_log_python_traceback_to_stderr_in_debug_mode(void)
   PyObject *exc, *value, *tb;
 
   PyErr_Fetch(&exc, &value, &tb);
+  if (!exc)
+    return;
 
   traceback_module = _py_do_import("traceback");
   if (!traceback_module)
@@ -112,7 +114,7 @@ _py_format_exception_text(gchar *buf, gsize buf_len)
 void
 _py_finish_exception_handling(void)
 {
-  _py_log_python_traceback_to_stderr_in_debug_mode();
+  _py_log_python_traceback_to_stderr();
   PyErr_Clear();
 }
 
@@ -186,7 +188,7 @@ _py_resolve_qualified_name(const gchar *name)
 
   if (!_split_fully_qualified_name(name, &module_name, &attribute_name))
     {
-      module_name = g_strdup("_syslogng");
+      module_name = g_strdup("_syslogng_main");
       attribute_name = g_strdup(name);
     }
 
@@ -425,3 +427,9 @@ _py_string_from_string(const gchar *str, gssize len)
     return PyBytes_FromString(str);
 #endif
 }
+
+void
+py_slng_generic_dealloc(PyObject *self)
+{
+  Py_TYPE(self)->tp_free(self);
+}
