@@ -27,7 +27,17 @@
 
 #include "python-module.h"
 
+typedef struct _PyLogMessage
+{
+  PyObject_HEAD
+  LogMessage *msg;
+} PyLogMessage;
+
+extern PyTypeObject py_log_message_type;
+
 PyObject *py_log_message_new(LogMessage *msg);
-void python_log_message_init(void);
+void py_log_message_init(void);
+
+int py_is_log_message(PyObject *obj);
 
 #endif
