@@ -29,6 +29,7 @@
 #include "python-logmsg.h"
 #include "python-logtemplate.h"
 #include "python-integerpointer.h"
+#include "python-logger.h"
 #include "python-source.h"
 #include "python-fetcher.h"
 #include "python-global-code-loader.h"
@@ -36,6 +37,9 @@
 
 #include "plugin.h"
 #include "plugin-types.h"
+#include "reloc.h"
+
+#include <stdlib.h>
 
 extern CfgParser python_parser;
 extern CfgParser python_parser_parser;
@@ -71,6 +75,20 @@ static Plugin python_plugins[] =
 };
 static gboolean interpreter_initialized = FALSE;
 
+static void
+_set_python_path(void)
+{
+  const gchar *current_python_path = getenv("PYTHONPATH");
+  GString *python_path = g_string_new(get_installation_path_for(SYSLOG_NG_PYTHON_MODULE_DIR));
+
+  if (current_python_path)
+    g_string_append_printf(python_path, ":%s", current_python_path);
+
+  setenv("PYTHONPATH", python_path->str, 1);
+
+  g_string_free(python_path, TRUE);
+}
+
 static void
 _py_init_interpreter(void)
 {
@@ -78,17 +96,18 @@ _py_init_interpreter(void)
     {
       python_debugger_append_inittab();
 
+      _set_python_path();
       Py_Initialize();
       py_init_argv();
 
       PyEval_InitThreads();
-      py_datetime_init();
       py_log_message_init();
       py_log_template_init();
       py_integer_pointer_init();
       py_log_source_init();
       py_log_fetcher_init();
       py_global_code_loader_init();
+      py_logger_init();
       PyEval_SaveThread();
 
       interpreter_initialized = TRUE;
