@@ -22,13 +22,16 @@
 
 #include <syslog-ng.h>
 #include <logmsg/logmsg.h>
-#include <template_lib.h>
+#include <cr_template.h>
 #include <apphook.h>
 #include <plugin.h>
-#include "date/date-parser.h"
 
+#include "cfg.h"
 
-MsgFormatOptions parse_options;
+#include <criterion/criterion.h>
+#include <stdlib.h>
+
+#include "msg_parse_lib.h"
 
 void
 stardate_assert(const gchar *msg_str, const int precision, const gchar *expected)
@@ -37,18 +40,38 @@ stardate_assert(const gchar *msg_str, const int precision, const gchar *expected
   LogMessage *logmsg = log_msg_new(msg_str, strlen(msg_str), NULL, &parse_options);
 
   char *template_command;
+  int ret_val;
   if (precision == -1)
-    asprintf(&template_command, "$(stardate $UNIXTIME)");
+    ret_val = asprintf(&template_command, "$(stardate $UNIXTIME)");
   else
-    asprintf(&template_command, "$(stardate --digits %d $UNIXTIME)", precision);
+    ret_val = asprintf(&template_command, "$(stardate --digits %d $UNIXTIME)", precision);
+
+  assert_false(ret_val == -1, "Memory allocation failed in asprintf.");
   assert_template_format_msg(template_command, expected, logmsg);
   free(template_command);
 
   log_msg_unref(logmsg);
 }
 
+
 void
-test_stardate(void)
+setup(void)
+{
+  app_startup();
+  init_template_tests();
+  cfg_load_module(configuration, "stardate");
+}
+
+void
+teardown(void)
+{
+  deinit_template_tests();
+  app_shutdown();
+}
+
+TestSuite(stardate, .init = setup, .fini = teardown);
+
+Test(stardate, test_stardate)
 {
   stardate_assert("2012-07-15T00:00:00", 1, "2012.5"); // 2012.01.01 + 365/2 day
   stardate_assert("2013-07-01T00:00:00", 2, "2013.49");
@@ -60,17 +83,3 @@ test_stardate(void)
   stardate_assert("2017-01-01T00:00:00", 0, "2017");
   stardate_assert("2018-12-01T00:00:00", 0, "2018"); // No rounding up!
 }
-
-int
-main(int argc G_GNUC_UNUSED, char *argv[] G_GNUC_UNUSED)
-{
-  app_startup();
-  init_template_tests();
-  cfg_load_module(configuration, "stardate");
-
-  test_stardate();
-
-  deinit_template_tests();
-  app_shutdown();
-  return 0;
-}
