@@ -92,24 +92,9 @@ libtool: compile: +gcc |\
 }
 
 ignore_submodule_gcc() {
-  ignore_submodule_gcc_mongo_c_driver "$@" |
-  ignore_submodule_gcc_rabbitmq_c |
   ignore_submodule_gcc_ivykis
 }
 
-ignore_submodule_gcc_mongo_c_driver() {
-  grep -vE -- "\<(\
-gcc( -std=gnu99)? -DPACKAGE_NAME=\\\\\"(libbson|mongo-c-driver)\\\\\"\
-)" "$@"
-}
-
-ignore_submodule_gcc_rabbitmq_c() {
-  grep -vE -- "\<(\
-gcc -std=gnu99 -DHAVE_CONFIG_H -I\. +-I\./librabbitmq |\
-gcc -std=gnu99 -DHAVE_CONFIG_H -I\. +-I\.\./\.\./\.\./\.\./modules/afamqp/rabbitmq-c \
-)" "$@"
-}
-
 ignore_submodule_gcc_ivykis() {
   grep -vE -- "\<(\
 gcc -DHAVE_CONFIG_H -I\. -I\.\./\.\. +-D_GNU_SOURCE -I\.\./\.\./src/include -I\.\./\.\./src/include |\
