@@ -55,14 +55,14 @@ _create_log_message(const gchar *log)
 }
 
 static LogMatcher *
-_construct_matcher(gint matcher_flags, LogMatcher *(*construct)(GlobalConfig *cfg, const LogMatcherOptions *options))
+_construct_matcher(gint matcher_flags, LogMatcher *(*construct)(const LogMatcherOptions *options))
 {
   LogMatcherOptions matcher_options;
 
   log_matcher_options_defaults(&matcher_options);
   matcher_options.flags = matcher_flags;
 
-  return construct(configuration, &matcher_options);
+  return construct(&matcher_options);
 }
 
 
@@ -154,12 +154,10 @@ Test(matcher, pcre_regexp, .description = "PCRE regexp")
                    _construct_matcher(LMF_GLOBAL, log_matcher_pcre_re_new));
 }
 
-Test(matcher, back_ref, .description = "back references are not portable, they work only on Linux")
+Test(matcher, back_ref)
 {
-#if __linux__
   testcase_replace("<155>2006-02-11T10:34:56+01:00 bzorp syslog-ng[23323]: wikiwiki", "(wiki)\\1", "", "",
                    _construct_matcher(LMF_STORE_MATCHES, log_matcher_pcre_re_new));
-#endif
 }
 
 Test(matcher, empty_global, .description = "empty match with global flag")
@@ -188,6 +186,12 @@ Test(matcher, empty_global, .description = "empty match with global flag")
                    _construct_matcher(LMF_GLOBAL, log_matcher_pcre_re_new));
 }
 
+Test(matcher, disable_jit)
+{
+  testcase_replace("<155>2006-02-11T10:34:56+01:00 bzorp syslog-ng[23323]: árvíztűrőtükörfúrógép", "árvíz",
+                   "favíz", "favíztűrőtükörfúrógép", _construct_matcher(LMF_DISABLE_JIT, log_matcher_pcre_re_new));
+}
+
 Test(matcher, string_match)
 {
   testcase_replace("<155>2006-02-11T10:34:56+01:00 bzorp syslog-ng[23323]: árvíztűrőtükörfúrógép", "árvíz",
